@@ -48,6 +48,7 @@
 #include "glade-catalog.h"
 #include "glade-app.h"
 #include "glade-paths.h"
+#include "glade-fixed.h"
 
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkstock.h>
@@ -63,13 +64,6 @@ struct _GladeAppPriv {
 	GladeEditor *editor;           /* See glade-editor */
 	GladeClipboard *clipboard;     /* See glade-clipboard */
 	GList *catalogs;               /* See glade-catalog */
-	GladeWidgetClass *add_class;   /* The GladeWidgetClass that we are about
-					* to add to a container. NULL if no
-					* class is to be added. This also has to
-					* be in sync with the depressed button
-					* in the GladePalette
-					*/
-	GladeWidgetClass *alt_class;
 	
 	GList *views;    /* A list of GladeProjectView item */
 	GList *projects; /* The list of Projects */
@@ -102,6 +96,7 @@ static guint glade_app_signals[LAST_SIGNAL] = { 0 };
 gchar *glade_pixmaps_dir = GLADE_PIXMAPSDIR;
 gchar *glade_catalogs_dir = GLADE_CATALOGSDIR;
 gchar *glade_modules_dir = GLADE_MODULESDIR;
+gchar *glade_plugins_dir = GLADE_PLUGINSDIR;
 gchar *glade_locale_dir = GLADE_LOCALEDIR;
 gchar *glade_icon_dir = GLADE_ICONDIR;
 gboolean glade_verbose = FALSE;
@@ -166,6 +161,7 @@ glade_app_dispose (GObject *app)
 static void
 glade_app_finalize (GObject *app)
 {
+
 #ifdef G_OS_WIN32 
 	g_free (glade_pixmaps_dir);
 	g_free (glade_catalogs_dir);
@@ -236,9 +232,9 @@ glade_app_refresh_undo_redo_button (GladeApp *app,
 	if ((project = glade_app_get_project ()) != NULL)
 	{
 		if (undo)
-			command = glade_command_next_undo_item (project);
+			command = glade_project_next_undo_item (project);
 		else
-			command = glade_command_next_redo_item (project);
+			command = glade_project_next_redo_item (project);
 	}
 
 	/* Change tooltips */
@@ -272,10 +268,10 @@ on_palette_button_clicked (GladePalette *palette, GladeApp *app)
 	GladeWidget *widget;
 
 	g_return_if_fail (GLADE_IS_PALETTE (palette));
-	class = palette->current;
+	class = glade_palette_get_current_item_class (palette);
 
 	/* class may be NULL if the selector was pressed */
-	if (class && g_type_is_a (class->type, GTK_TYPE_WINDOW))
+	if (class && class->toplevel)
 	{
 		widget = glade_command_create (class, NULL, NULL, app->priv->active_project);
 		
@@ -286,20 +282,8 @@ on_palette_button_clicked (GladePalette *palette, GladeApp *app)
 						    app->priv->accel_group);
 		}
 
-		glade_palette_unselect_widget (palette);
-		app->priv->add_class = NULL;
+		glade_palette_deselect_current_item (palette);
 	}
-	else if ((app->priv->add_class = class) != NULL)
-		app->priv->alt_class = class;
-}
-
-static void
-on_palette_catalog_changed (GladePalette *palette, GladeApp *app)
-{
-	g_return_if_fail (GLADE_IS_PALETTE (palette));
-
-	glade_palette_unselect_widget (palette);
-	app->priv->alt_class = app->priv->add_class = NULL;
 }
 
 static gboolean
@@ -347,29 +331,25 @@ glade_app_init (GladeApp *app)
 		initialized = TRUE;
 	}
 	app->priv = g_new0 (GladeAppPriv, 1);
-	
-	app->priv->add_class = NULL;
-	app->priv->alt_class = NULL;
+
 	app->priv->accel_group = NULL;
 	
 	/* Initialize app objects */
 	app->priv->catalogs = glade_catalog_load_all ();
 	
 	/* Create palette */
-	app->priv->palette = glade_palette_new (app->priv->catalogs);
+	app->priv->palette = (GladePalette *) glade_palette_new (app->priv->catalogs, GLADE_ITEM_ICON_ONLY);
 	g_object_ref (app->priv->palette);
 	gtk_object_sink (GTK_OBJECT (app->priv->palette));
 	gtk_widget_show_all (GTK_WIDGET (app->priv->palette));
 	g_signal_connect (G_OBJECT (app->priv->palette), "toggled",
 			  G_CALLBACK (on_palette_button_clicked), app);
-	g_signal_connect (G_OBJECT (app->priv->palette), "catalog-changed",
-			  G_CALLBACK (on_palette_catalog_changed), app);
 
 	/* Create Editor */
 	app->priv->editor = GLADE_EDITOR (glade_editor_new ());
 	g_object_ref (app->priv->editor);
 	gtk_object_sink (GTK_OBJECT (app->priv->editor));
-	gtk_widget_show_all (GTK_WIDGET (app->priv->editor));
+	gtk_widget_show (GTK_WIDGET (app->priv->editor));
 	glade_editor_refresh (app->priv->editor);
 	
 	/* Create clipboard */
@@ -683,20 +663,6 @@ glade_app_get_editor (void)
 	return app->priv->editor;
 }
 
-GladeWidgetClass *
-glade_app_get_add_class (void)
-{
-	GladeApp *app   = glade_app_get ();
-	return app->priv->add_class;
-}
-
-GladeWidgetClass *
-glade_app_get_alt_class (void)
-{
-	GladeApp         *app   = glade_app_get ();
-	return app->priv->alt_class;
-}
-
 GladePalette *
 glade_app_get_palette (void)
 {
@@ -775,6 +741,41 @@ glade_app_is_project_loaded (const gchar *project_path)
 	return loaded;
 }
 
+/**
+ * glade_app_get_project_by_path:
+ * @project_path: The path of an open project
+ *
+ * Finds an open project with @path
+ *
+ * Returns: A #GladeProject, or NULL if no such open project was found
+ */
+GladeProject*
+glade_app_get_project_by_path (const gchar *project_path)
+{
+	GladeApp *app;
+	GList    *l;
+	gchar *canonical_path;
+
+	if (project_path == NULL) return FALSE;
+
+	app = glade_app_get ();
+
+	canonical_path = glade_util_canonical_path (project_path);
+
+	for (l = app->priv->projects; l; l = l->next)
+	{
+		GladeProject *project = (GladeProject *) l->data;
+
+		if (project->path && strcmp (canonical_path, project->path) == 0) {
+			g_free (canonical_path);
+			return project;
+		}
+	}
+
+	g_free (canonical_path);
+
+	return NULL;
+}
 
 void
 glade_app_show_properties (gboolean raise)
@@ -1064,15 +1065,14 @@ glade_app_command_cut (void)
  * selection (the project must have only one object selected).
  */
 void
-glade_app_command_paste (void)
+glade_app_command_paste (GladePlaceholder *placeholder)
 {
 	GladeApp           *app;
 	GladeClipboard     *clipboard;
 	GList              *list;
-	GladeWidget        *widget = NULL;
+	GladeWidget        *widget = NULL, *parent;
 	gint                gtkcontainer_relations = 0;
-	GladePlaceholder   *placeholder;
-	GladeWidget        *parent;
+	GladeFixed	   *fixed = NULL;
 
 	app = glade_app_get();
 	if (app->priv->active_project == NULL)
@@ -1081,21 +1081,18 @@ glade_app_command_paste (void)
 	list      = glade_project_selection_get (app->priv->active_project);
 	clipboard = glade_app_get_clipboard ();
 
-	/* If there is a selection, dont use a placeholder to paste into */
-	placeholder = list ? NULL : glade_util_selected_placeholder ();
-
 	/* If there is a selection, paste in to the selected widget, otherwise
 	 * paste into the placeholder's parent.
 	 */
-	parent = list ? glade_widget_get_from_gobject (list->data) :
-		(placeholder ? glade_placeholder_get_parent (placeholder) : NULL);
-
-
+	parent = list ? glade_widget_get_from_gobject (list->data) : 
+		 (placeholder) ? glade_placeholder_get_parent (placeholder) : NULL;
+	
+	if (parent && GLADE_IS_FIXED (parent)) fixed = GLADE_FIXED (parent);
+		
 	/* Check if selection is good */
 	if ((list = glade_app_get_selection ()) != NULL)
 	{
-		if (placeholder == NULL &&
-		    g_list_length (list) != 1)
+		if (g_list_length (list) != 1)
 		{
 			glade_util_ui_message (glade_app_get_window(),
 					       GLADE_UI_INFO,
@@ -1142,9 +1139,10 @@ glade_app_command_paste (void)
 
 	g_assert (widget);
 
-	/* Check that GladeFixedManager will cope */
+	/* Check that GladeFixed will cope...
+ 	 */
 	if (GTK_WIDGET_TOPLEVEL (widget->object) == FALSE &&
-	    parent && parent->manager != NULL &&
+	    parent && fixed && fixed->use_placeholders == FALSE &&
 	    gtkcontainer_relations != 1) 
 	{
 		glade_util_ui_message (glade_app_get_window (), 
@@ -1155,7 +1153,7 @@ glade_app_command_paste (void)
 	}
 
 	/* Check that enough placeholders are available */
-	if (parent && parent->manager == NULL &&
+	if (parent && (fixed == NULL || fixed->use_placeholders) &&
 	    glade_util_count_placeholders (parent) < gtkcontainer_relations)
 	{
 		glade_util_ui_message (glade_app_get_window (), 
@@ -1264,7 +1262,7 @@ glade_app_command_undo (void)
 	GladeApp *app = glade_app_get();
 	if (app->priv->active_project)
 	{
-		glade_command_undo (app->priv->active_project);
+		glade_project_undo (app->priv->active_project);
 		glade_editor_refresh (app->priv->editor);
 		/* Update UI. */
 		glade_app_update_ui ();
@@ -1277,7 +1275,7 @@ glade_app_command_redo (void)
 	GladeApp *app = glade_app_get();
 	if (app->priv->active_project)
 	{
-		glade_command_redo (app->priv->active_project);
+		glade_project_redo (app->priv->active_project);
 		glade_editor_refresh (app->priv->editor);
 		/* Update UI. */
 		glade_app_update_ui ();
