@@ -41,8 +41,7 @@
 #include "glade-clipboard.h"
 #include "glade-signal.h"
 #include "glade-app.h"
-#include "glade-fixed-manager.h"
-
+#include "glade-fixed.h"
 
 /* Concerning placeholders: we do not hold any reference to placeholders,
  * placeholders that are supplied by the backend are not reffed, placeholders
@@ -55,8 +54,10 @@ typedef struct {
 	GladeWidget      *parent;
 	GladeProject     *project;
 	GladePlaceholder *placeholder;
+	gboolean          props_recorded;
 	GList            *pack_props;
-	gulong		 handler_id;
+	gchar            *special_type;
+	gulong            handler_id;
 } CommandData;
 
 static GObjectClass *parent_class = NULL;
@@ -115,13 +116,13 @@ glade_command_finalize (GObject *obj)
 }
 
 static gboolean
-glade_command_unifies (GladeCommand *this, GladeCommand *other)
+glade_command_unifies_impl (GladeCommand *this, GladeCommand *other)
 {
 	return FALSE;
 }
 
 static void
-glade_command_collapse (GladeCommand *this, GladeCommand *other)
+glade_command_collapse_impl (GladeCommand *this, GladeCommand *other)
 {
 	g_return_if_reached ();
 }
@@ -136,10 +137,10 @@ glade_command_class_init (GladeCommandClass *klass)
 
 	object_class->finalize = glade_command_finalize;
 
-	klass->undo_cmd = NULL;
-	klass->execute_cmd = NULL;
-	klass->unifies = glade_command_unifies;
-	klass->collapse = glade_command_collapse;
+	klass->undo            = NULL;
+	klass->execute         = NULL;
+	klass->unifies         = glade_command_unifies_impl;
+	klass->collapse        = glade_command_collapse_impl;
 }
 
 /* compose the _get_type function for GladeCommand */
@@ -163,8 +164,8 @@ func ## _class_init (gpointer parent_tmp, gpointer notused)		\
 	GladeCommandClass *parent = parent_tmp;				\
 	GObjectClass* object_class;					\
 	object_class = G_OBJECT_CLASS (parent);				\
-	parent->undo_cmd =  func ## _undo;				\
-	parent->execute_cmd =  func ## _execute;			\
+	parent->undo =  func ## _undo;					\
+	parent->execute =  func ## _execute;				\
 	parent->unifies =  func ## _unifies;				\
 	parent->collapse =  func ## _collapse;				\
 	object_class->finalize = func ## _finalize;			\
@@ -174,123 +175,96 @@ typedef struct {							\
 } type ## Class;							\
 static MAKE_TYPE(func, type, GLADE_TYPE_COMMAND)
 
-/**************************************************/
 
-
-GladeCommand *
-glade_command_next_undo_item (GladeProject *project)
+/**
+ * glade_command_execute:
+ * @command: A #GladeCommand
+ *
+ * Executes @command
+ *
+ * Returns: whether the command was successfully executed
+ */
+gboolean
+glade_command_execute (GladeCommand *command)
 {
-	GList *l;
-
-	g_return_val_if_fail (GLADE_IS_PROJECT (project), NULL);
-
-	if ((l = project->prev_redo_item) == NULL)
-		return NULL;
-
-	return GLADE_COMMAND (l->data);
-}
-
-GladeCommand *
-glade_command_next_redo_item (GladeProject *project)
-{
-	GList *l;
-
-	g_return_val_if_fail (GLADE_IS_PROJECT (project), NULL);
-
-	if ((l = project->prev_redo_item) == NULL)
-		return project->undo_stack ? 
-			GLADE_COMMAND (project->undo_stack->data) : NULL;
-	else
-		return l->next ? GLADE_COMMAND (l->next->data) : NULL;
-}
-
-static void
-glade_command_walk_back (GladeProject *project)
-{
-	if (project->prev_redo_item)
-		project->prev_redo_item = project->prev_redo_item->prev;
-}
-
-static void
-glade_command_walk_forward (GladeProject *project)
-{
-	if (project->prev_redo_item)
-		project->prev_redo_item = project->prev_redo_item->next;
-	else
-		project->prev_redo_item = project->undo_stack;
+	g_return_val_if_fail (GLADE_IS_COMMAND (command), FALSE);
+	return GLADE_COMMAND_GET_CLASS (command)->execute (command);
 }
 
 
 /**
  * glade_command_undo:
- * @project: a #GladeProject
+ * @command: A #GladeCommand
  *
- * Undoes the last command performed in @project.
+ * Undo the effects of @command
+ *
+ * Returns whether the command was successfully reversed
  */
-void
-glade_command_undo (GladeProject *project)
+gboolean
+glade_command_undo (GladeCommand *command)
 {
-	GladeCommand *cmd, *next_cmd;
-
-	g_return_if_fail (GLADE_IS_PROJECT (project));
-
-
-	while ((cmd = glade_command_next_undo_item (project)) != NULL)
-	{
-		GLADE_COMMAND_GET_CLASS (cmd)->undo_cmd (cmd);
-
-		glade_command_walk_back (project);
-
-		if ((next_cmd = glade_command_next_undo_item (project)) != NULL &&
-		    (next_cmd->group_id == 0 || next_cmd->group_id != cmd->group_id))
-			break;
-	}
+	g_return_val_if_fail (GLADE_IS_COMMAND (command), FALSE);
+	return GLADE_COMMAND_GET_CLASS (command)->undo (command);
 }
 
 /**
- * glade_command_redo:
- * @project: a #GladeProject
+ * glade_command_unifies:
+ * @command: A #GladeCommand
+ * @other: another #GladeCommand
  *
- * Redoes the last undone command in @project.
+ * Checks whether @command and @other can be unified
+ * to make one single command.
+ *
+ * Returns: whether they can be unified.
+ */
+gboolean
+glade_command_unifies (GladeCommand *command,
+		       GladeCommand *other)
+{
+	g_return_val_if_fail (command, FALSE);
+	return GLADE_COMMAND_GET_CLASS (command)->unifies (command, other);
+}
+
+/**
+ * glade_command_collapse:
+ * @command: A #GladeCommand
+ * @other: another #GladeCommand
+ *
+ * Merges @other into @command, so that @command now
+ * covers both commands and @other can be dispensed with.
  */
 void
-glade_command_redo (GladeProject *project)
+glade_command_collapse (GladeCommand  *command,
+			GladeCommand  *other)
 {
-	GladeCommand *cmd, *next_cmd;
-	
-	g_return_if_fail (GLADE_IS_PROJECT (project));
-
-
-	while ((cmd = glade_command_next_redo_item (project)) != NULL)
-	{
-		GLADE_COMMAND_GET_CLASS (cmd)->execute_cmd (cmd);
-
-		glade_command_walk_forward (project);
-
-		if ((next_cmd = glade_command_next_redo_item (project)) != NULL &&
-		    (next_cmd->group_id == 0 || next_cmd->group_id != cmd->group_id))
-			break;
-	}
+	g_return_if_fail (command);
+	GLADE_COMMAND_GET_CLASS (command)->collapse (command, other);
 }
 
 /**
  * glade_command_push_group:
- * @description: The collective desctiption of the command group.
+ * @fmt:         The collective desctiption of the command group.
  *               only the description of the first group on the 
  *               stack is used when embedding groups.
+ * @...: args to the format string.
  *
  * Marks the begining of a group.
- *
  */
 void
-glade_command_push_group (const gchar *description)
+glade_command_push_group (const gchar *fmt, ...)
 {
-	g_return_if_fail (description);
+	va_list         args;
+
+	g_return_if_fail (fmt);
 
 	/* Only use the description for the first group.
 	 */
 	if (gc_group_depth++ == 0)
-		gc_group_description = g_strdup (description);
+	{
+		va_start (args, fmt);
+		gc_group_description = g_strdup_vprintf (fmt, args);
+		va_end (args);
+	}
 }
 
 /**
@@ -324,60 +298,6 @@ glade_command_check_group (GladeCommand *cmd)
 	}
 }
 
-static void
-glade_command_push_undo (GladeProject *project, GladeCommand *cmd)
-{
-	GList* tmp_redo_item;
-
-	g_return_if_fail (GLADE_IS_PROJECT (project));
-	g_return_if_fail (GLADE_IS_COMMAND (cmd));
-
-	/* If there are no "redo" items, and the last "undo" item unifies with
-	   us, then we collapse the two items in one and we're done */
-	if (project->prev_redo_item != NULL && project->prev_redo_item->next == NULL)
-	{
-		GladeCommand* cmd1 = project->prev_redo_item->data;
-		GladeCommandClass* klass = GLADE_COMMAND_GET_CLASS (cmd1);
-		
-		if (klass->unifies (cmd1, cmd))
-		{
-			klass->collapse (cmd1, cmd);
-			g_object_unref (cmd);
-			return;
-		}
-	}
-
-	/* We should now free all the "redo" items */
-	tmp_redo_item = g_list_next (project->prev_redo_item);
-	while (tmp_redo_item)
-	{
-		g_assert (tmp_redo_item->data);
-		g_object_unref (G_OBJECT (tmp_redo_item->data));
-		tmp_redo_item = g_list_next (tmp_redo_item);
-	}
-
-	if (project->prev_redo_item)
-	{
-		g_list_free (g_list_next (project->prev_redo_item));
-		project->prev_redo_item->next = NULL;
-	}
-	else
-	{
-		g_list_free (project->undo_stack);
-		project->undo_stack = NULL;
-	}
-
-	/* and then push the new undo item */
-	project->undo_stack = g_list_append (project->undo_stack, cmd);
-
-	if (project->prev_redo_item == NULL)
-		project->prev_redo_item = project->undo_stack;
-	else
-		project->prev_redo_item = g_list_next (project->prev_redo_item);
-
-	glade_app_update_ui ();
-}
-
 /**************************************************/
 /*******     GLADE_COMMAND_SET_PROPERTY     *******/
 /**************************************************/
@@ -386,8 +306,10 @@ glade_command_push_undo (GladeProject *project, GladeCommand *cmd)
  * encapsulate a "set property" operation */
 
 typedef struct {
-	GladeCommand parent;
-	GList *sdata;
+	GladeCommand  parent;
+	gboolean      set_once;
+	gboolean      undo;
+	GList        *sdata;
 } GladeCommandSetProperty;
 
 /* standard macros */
@@ -417,29 +339,78 @@ glade_command_set_property_execute (GladeCommand *cmd)
 
 	g_return_val_if_fail (me != NULL, FALSE);
 
+	if (me->set_once != FALSE)
+		glade_property_push_superuser ();
+
 	for (l = me->sdata; l; l = l->next)
 	{
 		GValue new_value = { 0, };
 		GCSetPropData *sdata = l->data;
 
 		g_value_init (&new_value, G_VALUE_TYPE (sdata->new_value));
-		g_value_copy (sdata->new_value, &new_value);
 		
-		/* store the current value for undo */
-		if (sdata->old_value)
-		{
-			g_value_copy (sdata->old_value, sdata->new_value);
-			g_value_unset (sdata->old_value);
-			sdata->old_value = (g_free (sdata->old_value), NULL);
-		}
+		if (me->undo)
+			g_value_copy (sdata->old_value, &new_value);
 		else
-			g_value_copy (sdata->property->value, sdata->new_value);
+			g_value_copy (sdata->new_value, &new_value);
+
+#if 0
+		{
+			gchar *str =
+				glade_property_class_make_string_from_gvalue 
+				(sdata->property->class, &new_value);
+
+			g_print ("Setting %s property of %s to %s (sumode: %d)\n",
+				 sdata->property->class->id,
+				 sdata->property->widget->name,
+				 str, glade_property_superuser ());
+
+			g_free (str);
+		}
+#endif
+
+		/* Packing properties need to be refreshed here since
+		 * they are reset when they get added to containers.
+		 */
+		if (sdata->property->class->packing)
+		{
+			GladeProperty *tmp_prop;
+
+			tmp_prop = glade_widget_get_pack_property
+				(sdata->property->widget, sdata->property->class->id);
+
+			if (sdata->property != tmp_prop)
+			{
+				g_object_unref (sdata->property);
+				sdata->property = g_object_ref (tmp_prop);
+				
+			}
+		}
 
 		glade_property_set_value (sdata->property, &new_value);
+
+		if (!me->set_once)
+		{
+			/* If some verify functions didnt pass on 
+			 * the first go.. we need to record the actual
+			 * properties here.
+			 */
+			g_value_copy (sdata->property->value, 
+				      sdata->new_value);
+		}
+
+
 		g_value_unset (&new_value);
 
 
 	}
+
+	if (me->set_once != FALSE)
+		glade_property_pop_superuser ();
+
+	me->set_once = TRUE;
+	me->undo     = !me->undo;
+
 	return TRUE;
 }
 
@@ -475,24 +446,42 @@ glade_command_set_property_finalize (GObject *obj)
 static gboolean
 glade_command_set_property_unifies (GladeCommand *this, GladeCommand *other)
 {
+       GladeCommandSetProperty *cmd1,   *cmd2;
+       GCSetPropData           *pdata1, *pdata2;
+       GList                   *list, *l;
 
-       /* XXX FIXME: This doesn't make sence with the reset dialog, when resetting properties
-	* they should not unify, also; setting multiple properties should unify as well as long
-	* as the same group is changing (this will help with drag/resize widget control).
-	*
-	*/
-       GladeCommandSetProperty *cmd1;
-       GladeCommandSetProperty *cmd2;
-
-       if (GLADE_IS_COMMAND_SET_PROPERTY (this) && GLADE_IS_COMMAND_SET_PROPERTY (other))
+       if (GLADE_IS_COMMAND_SET_PROPERTY (this) && 
+	   GLADE_IS_COMMAND_SET_PROPERTY (other))
        {
                cmd1 = (GladeCommandSetProperty*) this;
                cmd2 = (GladeCommandSetProperty*) other;
-	       
-               return (g_list_length (cmd1->sdata) == 1 &&
-                      g_list_length (cmd2->sdata) == 1 &&
-                       ((GCSetPropData *)cmd1->sdata->data)->property == 
-                       ((GCSetPropData *)cmd2->sdata->data)->property);
+
+	       if (g_list_length (cmd1->sdata) != 
+		   g_list_length (cmd2->sdata))
+		       return FALSE;
+
+	       for (list = cmd1->sdata; list; list = list->next)
+	       {
+		       pdata1 = list->data;
+		       for (l = cmd2->sdata; l; l = l->next)
+		       {
+			       pdata2 = l->data;
+
+			       if (pdata1->property->widget == pdata2->property->widget &&
+				   glade_property_class_match (pdata1->property->class,
+							       pdata2->property->class))
+				       break;
+		       }
+		       
+		       /* If both lists are the same length, and one class type
+			* is not found in the other list, then we cant unify.
+			*/
+		       if (l == NULL)
+			       return FALSE;
+
+	       }
+
+	       return TRUE;
        }
        return FALSE;
 }
@@ -500,9 +489,37 @@ glade_command_set_property_unifies (GladeCommand *this, GladeCommand *other)
 static void
 glade_command_set_property_collapse (GladeCommand *this, GladeCommand *other)
 {
+	GladeCommandSetProperty *cmd1,   *cmd2;
+	GCSetPropData           *pdata1, *pdata2;
+	GList                   *list, *l;
+	
 	g_return_if_fail (GLADE_IS_COMMAND_SET_PROPERTY (this) &&
 			  GLADE_IS_COMMAND_SET_PROPERTY (other));
+	
+	cmd1 = (GladeCommandSetProperty*) this;
+	cmd2 = (GladeCommandSetProperty*) other;
 
+
+	for (list = cmd1->sdata; list; list = list->next)
+	{
+		pdata1 = list->data;
+		for (l = cmd2->sdata; l; l = l->next)
+		{
+			pdata2 = l->data;
+			
+			if (glade_property_class_match (pdata1->property->class,
+							pdata2->property->class))
+			{
+				/* Merge the GCSetPropData structs manually here
+				 */
+				g_value_copy (pdata2->new_value, pdata1->new_value);
+				break;
+			}
+		}
+	}
+
+	/* Set the description
+	 */
 	g_free (this->description);
 	this->description = other->description;
 	other->description = NULL;
@@ -573,7 +590,7 @@ glade_command_set_properties_list (GladeProject *project, GList *props)
 
 	/* Push onto undo stack only if it executes successfully. */
 	if (glade_command_set_property_execute (GLADE_COMMAND (me)))
-		glade_command_push_undo (GLADE_PROJECT (project),
+		glade_project_push_undo (GLADE_PROJECT (project),
 					 GLADE_COMMAND (me));
 	else
 		/* No leaks on my shift! */
@@ -629,11 +646,32 @@ glade_command_set_properties (GladeProperty *property, const GValue *old_value,
 }
 
 void
-glade_command_set_property (GladeProperty *property, const GValue* pvalue)
+glade_command_set_property_value (GladeProperty *property, const GValue* pvalue)
 {
+
+	/* Dont generate undo/redo items for unchanging property values.
+	 */
+	if (glade_property_equals_value (property, pvalue))
+		return;
+
 	glade_command_set_properties (property, property->value, pvalue, NULL);
 }
 
+void
+glade_command_set_property (GladeProperty *property, ...)
+{
+	GValue *value;
+	va_list args;
+	
+	g_return_if_fail (GLADE_IS_PROPERTY (property));
+
+	va_start (args, property);
+	value = glade_property_class_make_gvalue_from_vl (property->class, args);
+	va_end (args);
+	
+	glade_command_set_property_value (property, value);
+}
+
 /**************************************************/
 /*******       GLADE_COMMAND_SET_NAME       *******/
 /**************************************************/
@@ -763,7 +801,7 @@ glade_command_set_name (GladeWidget *widget, const gchar* name)
 	glade_command_check_group (GLADE_COMMAND (me));
 
 	if (glade_command_set_name_execute (GLADE_COMMAND (me)))
-		glade_command_push_undo (GLADE_PROJECT (widget->project), GLADE_COMMAND (me));
+		glade_project_push_undo (GLADE_PROJECT (widget->project), GLADE_COMMAND (me));
 	else
 		g_object_unref (G_OBJECT (me));
 }
@@ -778,7 +816,6 @@ typedef struct {
 	GList            *widgets;
 	gboolean          create;
 	gboolean          from_clipboard;
-	gboolean          props_recorded;
 } GladeCommandCreateDelete;
 
 GLADE_MAKE_COMMAND (GladeCommandCreateDelete, glade_command_create_delete);
@@ -835,28 +872,12 @@ glade_command_create_execute (GladeCommandCreateDelete *me)
 		if (cdata->parent)
 		{
 			if (cdata->placeholder)
-			{
 				glade_widget_replace
 					(cdata->parent, 
 					 G_OBJECT (cdata->placeholder), 
 					 cdata->widget->object);
-			}
-			else if (cdata->parent->manager != NULL)
-			{
-				glade_fixed_manager_add_child
-					(cdata->parent->manager, cdata->widget, FALSE);
-			}
 			else
-			{
-				glade_widget_set_parent (cdata->widget, cdata->parent);
-
-				glade_widget_class_container_add 
-					(cdata->parent->widget_class,
-					 cdata->parent->object,
-					 cdata->widget->object);
-
-				glade_widget_set_packing_properties (cdata->widget, cdata->parent);
-			}
+				glade_widget_add_child (cdata->parent, cdata->widget, FALSE);
 
 			/* Now that we've added, apply any packing props if nescisary. */
 			for (l = cdata->pack_props; l; l = l->next)
@@ -872,7 +893,7 @@ glade_command_create_execute (GladeCommandCreateDelete *me)
 				g_value_unset (&value);
 			}
 				
-			if (me->props_recorded == FALSE) 
+			if (cdata->props_recorded == FALSE) 
 			{
 				/* Save the packing properties after the initial creation.
 				 * (this will be the defaults returned by the container
@@ -890,7 +911,7 @@ glade_command_create_execute (GladeCommandCreateDelete *me)
 
 				/* Mark the properties as recorded
 				 */
-				me->props_recorded = TRUE;
+				cdata->props_recorded = TRUE;
 			}
 
 		}
@@ -938,16 +959,11 @@ glade_command_delete_execute (GladeCommandCreateDelete *me)
 				glade_widget_replace
 					(cdata->parent, cdata->widget->object, 
 					 G_OBJECT (cdata->placeholder));
-			else if (cdata->parent->manager != NULL)
-				glade_fixed_manager_remove_child
-					(cdata->parent->manager, cdata->widget);
 			else
-				glade_widget_class_container_remove (cdata->parent->widget_class,
-								     cdata->parent->object,
-								     cdata->widget->object);
+				glade_widget_remove_child (cdata->parent, cdata->widget);
 		}
 
-		if (me->from_clipboard == TRUE) 
+		if (me->from_clipboard != FALSE) 
 			wlist = g_list_prepend (wlist, cdata->widget);
 		else
 			glade_project_remove_object 
@@ -1050,8 +1066,6 @@ glade_command_delete (GList *widgets)
 	me->create         = FALSE;
 	me->from_clipboard = 
 		(g_list_find (clipboard->selection, widgets->data) != NULL);
-	me->props_recorded = TRUE; // Dont record props in create_execute
-
 
 	/* internal children cannot be deleted. Notify the user. */
 	for (list = widgets; list && list->data; list = list->next)
@@ -1077,9 +1091,9 @@ glade_command_delete (GList *widgets)
 		if (widget->internal)
 			g_critical ("Internal widget in Delete");
 
-		/* !manager here */
+		/* !fixed here */
 		if (cdata->parent != NULL &&
-		    cdata->parent->manager == NULL &&
+		    GLADE_IS_FIXED (cdata->parent) == FALSE &&
 		    glade_util_gtkcontainer_relation 
 		    (cdata->parent, cdata->widget))
 		{
@@ -1088,6 +1102,10 @@ glade_command_delete (GList *widgets)
 		}
 		me->widgets = g_list_prepend (me->widgets, cdata);
 
+		/* Dont record props in create_execute (whether or not we actually
+		 * record any props here
+		 */
+		cdata->props_recorded = TRUE; 
 
 		/* Record packing props if not deleted from the clipboard */
 		if (me->from_clipboard == FALSE)
@@ -1113,7 +1131,7 @@ glade_command_delete (GList *widgets)
 	glade_command_check_group (GLADE_COMMAND (me));
 
 	if (glade_command_create_delete_execute (GLADE_COMMAND (me)))
-		glade_command_push_undo (GLADE_PROJECT (widget->project), 
+		glade_project_push_undo (GLADE_PROJECT (widget->project), 
 					 GLADE_COMMAND (me));
 	else
 		g_object_unref (G_OBJECT (me));
@@ -1142,7 +1160,7 @@ glade_command_create (GladeWidgetClass *class,
 
 	g_return_val_if_fail (class != NULL, NULL);
 	g_return_val_if_fail (GLADE_IS_PROJECT (project), NULL);
-	if (g_type_is_a (class->type, GTK_TYPE_WINDOW) == FALSE)
+	if (class->toplevel == FALSE)
 		g_return_val_if_fail (GLADE_IS_WIDGET (parent), NULL);
 
  	me            = g_object_new (GLADE_COMMAND_CREATE_DELETE_TYPE, NULL);
@@ -1155,10 +1173,10 @@ glade_command_create (GladeWidgetClass *class,
 	
 	me->widgets = g_list_append (me->widgets, cdata);
 
-	if (parent && parent->manager != NULL)
-		widget = glade_fixed_manager_create_child (parent->manager, class);
-	else
-		widget = glade_widget_new (parent, class, project, TRUE);
+	widget = glade_widget_class_create_widget (class, TRUE,
+						   "parent", parent, 
+						   "project", project, 
+						   NULL);
 
 	/* widget may be null, e.g. the user clicked cancel on a query */
 	if ((cdata->widget = widget) == NULL)
@@ -1180,7 +1198,7 @@ glade_command_create (GladeWidgetClass *class,
 	glade_command_check_group (GLADE_COMMAND (me));
 
 	if (glade_command_create_delete_execute (GLADE_COMMAND (me)))
-		glade_command_push_undo (project, GLADE_COMMAND (me));
+		glade_project_push_undo (project, GLADE_COMMAND (me));
 	else
 		g_object_unref (G_OBJECT (me));
 
@@ -1204,8 +1222,8 @@ typedef struct {
 	GladeProject          *project;
 	GList                 *widgets;
 	GladeCutCopyPasteType  type;
+	GladeCutCopyPasteType  original_type;
 	gboolean               from_clipboard;
-	gboolean               props_recorded;
 } GladeCommandCutCopyPaste;
 
 
@@ -1222,6 +1240,7 @@ glade_command_paste_execute (GladeCommandCutCopyPaste *me)
 	GladeProject       *active_project = glade_app_get_project ();
 	CommandData        *cdata;
 	GList              *list, *remove = NULL, *l;
+	gchar              *special_child_type;
 
 	if (me->widgets)
 	{
@@ -1234,30 +1253,37 @@ glade_command_paste_execute (GladeCommandCutCopyPaste *me)
 
 			if (cdata->parent != NULL)
 			{
+				/* Prepare special-child-type for the paste. */
+				if (cdata->props_recorded == FALSE)
+				{
+					/* Clear it the first time */
+					g_object_set_data (cdata->widget->object,
+							   "special-child-type", NULL);
+				}
+				else
+				{
+					g_object_set_data_full (cdata->widget->object, 
+								"special-child-type",
+								g_strdup (cdata->special_type), 
+								g_free);
+				}
+
 				/* glade_command_paste ganauntees that if 
 				 * there we are pasting to a placeholder, 
 				 * there is only one widget.
 				 */
 				if (cdata->placeholder)
+				{
 					glade_widget_replace
 						(cdata->parent,
 						 G_OBJECT (cdata->placeholder),
 						 cdata->widget->object);
-				else if (cdata->parent->manager != NULL) 
-					/* Paste at mouse position only once */
-					glade_fixed_manager_add_child (cdata->parent->manager, cdata->widget,
-								       me->props_recorded == FALSE);
+				}
 				else
 				{
-					glade_widget_set_parent (cdata->widget, 
-								 cdata->parent);
-
-					glade_widget_class_container_add
-						(cdata->parent->widget_class,
-						 cdata->parent->object,
-						 cdata->widget->object);
-
-					glade_widget_set_packing_properties (cdata->widget, cdata->parent);
+					glade_widget_add_child (cdata->parent,
+								cdata->widget, 
+								cdata->props_recorded == FALSE);
 				}
 
 				/* Now that we've added, apply any packing props if nescisary. */
@@ -1274,7 +1300,7 @@ glade_command_paste_execute (GladeCommandCutCopyPaste *me)
 					g_value_unset (&value);
 				}
 				
-				if (me->props_recorded == FALSE) 
+				if (cdata->props_recorded == FALSE) 
 				{
 					/* Save the packing properties after the initial paste.
 					 * (this will be the defaults returned by the container
@@ -1283,16 +1309,29 @@ glade_command_paste_execute (GladeCommandCutCopyPaste *me)
 					 * Otherwise this recorded marker was set when cutting
 					 */
 					g_assert (cdata->pack_props == NULL);
-					for (l = cdata->widget->packing_properties; l; l = l->next)
+					for (l = cdata->widget->packing_properties; 
+					     l; l = l->next)
 						cdata->pack_props = 
-							g_list_prepend (cdata->pack_props,
-									glade_property_dup (GLADE_PROPERTY (l->data),
-											    cdata->widget));
+							g_list_prepend
+							(cdata->pack_props,
+							 glade_property_dup
+							 (GLADE_PROPERTY (l->data),
+							  cdata->widget));
+
+
+					/* Record the special-type here after replacing */
+					if ((special_child_type = 
+					     g_object_get_data (cdata->widget->object, 
+								"special-child-type")) != NULL)
+					{
+						g_free (cdata->special_type);
+						cdata->special_type = g_strdup (special_child_type);
+					}
 
 
 					/* Mark the properties as recorded
 					 */
-					me->props_recorded = TRUE;
+					cdata->props_recorded = TRUE;
 				}
 				
 				
@@ -1330,12 +1369,27 @@ glade_command_cut_execute (GladeCommandCutCopyPaste *me)
 {
 	CommandData        *cdata;
 	GList              *list, *add = NULL;
+	gchar              *special_child_type;
 
 	for (list = me->widgets; list && list->data; list = list->next)
 	{
 		cdata = list->data;
 		add   = g_list_prepend (add, cdata->widget);
 
+		if (me->original_type == GLADE_CUT)
+		{
+			if ((special_child_type = 
+			     g_object_get_data (cdata->widget->object, 
+						"special-child-type")) != NULL)
+			{
+				g_free (cdata->special_type);
+				cdata->special_type = g_strdup (special_child_type);
+			}
+		}
+		else
+			g_object_set_data (cdata->widget->object,
+					   "special-child-type", NULL);
+
 		if (cdata->parent)
 		{
 			if (cdata->placeholder)
@@ -1343,15 +1397,11 @@ glade_command_cut_execute (GladeCommandCutCopyPaste *me)
 					(cdata->parent,
 					 cdata->widget->object,
 					 G_OBJECT (cdata->placeholder));
-			else if (cdata->parent->manager != NULL) 
-				glade_fixed_manager_remove_child
-					(cdata->parent->manager, cdata->widget);
 			else
-				glade_widget_class_container_remove
-					(cdata->parent->widget_class,
-					 cdata->parent->object,
-					 cdata->widget->object);
+				glade_widget_remove_child (cdata->parent, cdata->widget);
 		}
+
+		g_object_set_data (cdata->widget->object, "special-child-type", NULL);
 		
 		glade_widget_hide (cdata->widget);
 		glade_project_remove_object (GLADE_PROJECT (cdata->widget->project),
@@ -1498,9 +1548,10 @@ glade_command_cut_copy_paste_common (GList                 *widgets,
 {
 	GladeCommandCutCopyPaste *me;
 	CommandData              *cdata;
-	GladeWidget              *widget = NULL, *child;
+	GladeWidget              *widget = NULL;
 	GList                    *l, *list, *children, *placeholders = NULL;
 	gchar                    *fmt = NULL;
+	GtkWidget                *child;
 
 	g_return_if_fail (widgets && widgets->data);
 	g_return_if_fail (parent == NULL || GLADE_IS_WIDGET (parent));
@@ -1528,6 +1579,7 @@ glade_command_cut_copy_paste_common (GList                 *widgets,
 		me->project = glade_widget_get_project (widget);
 
 	me->type           = type;
+	me->original_type  = type;
 	me->from_clipboard = (type == GLADE_PASTE);
 	GLADE_COMMAND (me)->description = 
 		g_strdup_printf (fmt, g_list_length (widgets) == 1 ? 
@@ -1569,7 +1621,7 @@ glade_command_cut_copy_paste_common (GList                 *widgets,
 		/* Placeholder */
 		if (type == GLADE_CUT)
 		{
-			if (cdata->parent && cdata->parent->manager == NULL &&
+			if (cdata->parent && GLADE_IS_FIXED (cdata->parent) == FALSE &&
 			    glade_util_gtkcontainer_relation
 			    (cdata->parent, cdata->widget))
 			{
@@ -1585,11 +1637,19 @@ glade_command_cut_copy_paste_common (GList                 *widgets,
 			glade_command_placeholder_connect (cdata, placeholder);
 		}
 		else if (type == GLADE_PASTE && cdata->parent &&
-			 cdata->parent->manager == NULL &&
 			 glade_util_gtkcontainer_relation (cdata->parent, widget))
 		{
-			if ((children = glade_widget_class_container_get_children
-			     (cdata->parent->widget_class, cdata->parent->object)) != NULL)
+			GtkContainer *cont = GTK_CONTAINER (cdata->parent->object);
+			
+			child = glade_util_get_placeholder_from_pointer (cont);
+			if (child && g_list_find (placeholders, child) == NULL)
+			{
+				placeholders = g_list_append (placeholders, child);
+				glade_command_placeholder_connect
+						(cdata, GLADE_PLACEHOLDER (child));
+			}
+			else if ((children = glade_widget_class_container_get_children
+				 (cdata->parent->widget_class, cdata->parent->object)) != NULL)
 			{
 				for (l = children; l && l->data; l = l->next)
 				{
@@ -1618,7 +1678,7 @@ glade_command_cut_copy_paste_common (GList                 *widgets,
 			 * packing props when we undo a CUT command; so we have to 
 			 * mark them "recorded" too.
 			 */
-			me->props_recorded = TRUE;
+			cdata->props_recorded = TRUE;
 			for (l = cdata->widget->packing_properties; l; l = l->next)
 				cdata->pack_props = 
 					g_list_prepend (cdata->pack_props,
@@ -1643,7 +1703,7 @@ glade_command_cut_copy_paste_common (GList                 *widgets,
 	 * Push it onto the undo stack only on success
 	 */
 	if (glade_command_cut_copy_paste_execute (GLADE_COMMAND (me)))
-		glade_command_push_undo 
+		glade_project_push_undo 
 			(glade_app_get_project(),
 			 GLADE_COMMAND (me));
 	else
@@ -1728,7 +1788,6 @@ static void
 glade_command_add_signal_finalize (GObject *obj)
 {
 	GladeCommandAddSignal *cmd = GLADE_COMMAND_ADD_SIGNAL (obj);
-	glade_signal_free (cmd->signal);
 
 	g_object_unref (cmd->widget);
 
@@ -1817,7 +1876,7 @@ glade_command_add_remove_change_signal (GladeWidget       *glade_widget,
 	glade_command_check_group (GLADE_COMMAND (me));
 
 	if (glade_command_add_signal_execute (cmd))
-		glade_command_push_undo (GLADE_PROJECT (glade_widget->project), cmd);
+		glade_project_push_undo (GLADE_PROJECT (glade_widget->project), cmd);
 	else
 		g_object_unref (G_OBJECT (me));
 }
