@@ -29,6 +29,7 @@
 #include <glib.h>
 #include <glib/gi18n-lib.h>
 #include <libxml/parser.h>
+#include <time.h>
 
 #include "glade-parser.h"
 #include "glade.h"
@@ -125,9 +126,9 @@ create_widget_info(GladeInterface *interface, const xmlChar **attrs)
 
     for (i = 0; attrs && attrs[i] != NULL; i += 2) {
         if (!xmlStrcmp(attrs[i], BAD_CAST("class")))
-            info->classname = alloc_string (interface, CAST_BAD(attrs[i+1]));
+            info->classname = glade_xml_alloc_string (interface, CAST_BAD(attrs[i+1]));
             else if (!xmlStrcmp(attrs[i], BAD_CAST("id")))
-                info->name = alloc_string (interface, CAST_BAD(attrs[i+1]));
+                info->name = glade_xml_alloc_string (interface, CAST_BAD(attrs[i+1]));
             else
                 g_warning("unknown attribute `%s' for <widget>.", attrs[i]);
     }
@@ -233,9 +234,9 @@ handle_atk_action(GladeParseState *state, const xmlChar **attrs)
 
     for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 	if (!xmlStrcmp(attrs[i], BAD_CAST("action_name")))
-	    info.action_name = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info.action_name = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else if (!xmlStrcmp(attrs[i], BAD_CAST("description")))
-	    info.description = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info.description = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else
 	    g_warning("unknown attribute `%s' for <action>.", attrs[i]);
     }
@@ -259,9 +260,9 @@ handle_atk_relation(GladeParseState *state, const xmlChar **attrs)
 
     for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 	if (!xmlStrcmp(attrs[i], BAD_CAST("target")))
-	    info.target = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info.target = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else if (!xmlStrcmp(attrs[i], BAD_CAST("type")))
-	    info.type = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info.type = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else
 	    g_warning("unknown attribute `%s' for <signal>.", attrs[i]);
     }
@@ -286,15 +287,15 @@ handle_signal(GladeParseState *state, const xmlChar **attrs)
     info.after = FALSE;
     for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 	if (!xmlStrcmp(attrs[i], BAD_CAST("name")))
-	    info.name = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info.name = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else if (!xmlStrcmp(attrs[i], BAD_CAST("handler")))
-	    info.handler = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info.handler = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else if (!xmlStrcmp(attrs[i], BAD_CAST("after")))
 	    info.after = attrs[i+1][0] == 'y';
 	else if (!xmlStrcmp(attrs[i], BAD_CAST("lookup")))
 	    info.lookup = attrs[i+1][0] == 'y';
 	else if (!xmlStrcmp(attrs[i], BAD_CAST("object")))
-	    info.object = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info.object = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else if (!xmlStrcmp(attrs[i], BAD_CAST("last_modification_time")))
 	    /* Do nothing. */;
 	else
@@ -378,7 +379,7 @@ handle_accel(GladeParseState *state, const xmlChar **attrs)
                } else
                    pos++;
 	} else if (!xmlStrcmp(attrs[i], BAD_CAST("signal")))
-	    info.signal = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info.signal = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else
 	    g_warning("unknown attribute `%s' for <accelerator>.", attrs[i]);
     }
@@ -416,7 +417,7 @@ handle_child(GladeParseState *state, const xmlChar **attrs)
 
     for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 	if (!xmlStrcmp(attrs[i], BAD_CAST("internal-child")))
-	    info->internal_child = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info->internal_child = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else
 	    g_warning("unknown attribute `%s' for <child>.", attrs[i]);
     }
@@ -499,7 +500,7 @@ glade_parser_start_element(GladeParseState *state,
 		    iface->requires = g_renew(gchar *, iface->requires,
 					      iface->n_requires);
 		    iface->requires[iface->n_requires-1] =
-			alloc_string(iface, CAST_BAD(attrs[i+1]));
+			glade_xml_alloc_string(iface, CAST_BAD(attrs[i+1]));
 		} else
 		    g_warning("unknown attribute `%s' for <requires>.",
 			      attrs[i]);
@@ -547,14 +548,14 @@ glade_parser_start_element(GladeParseState *state,
 	    state->translate_prop = FALSE;
 	    for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 		if (!xmlStrcmp(attrs[i], BAD_CAST("name")))
-		    state->prop_name = alloc_propname(state->interface,
+		    state->prop_name = glade_xml_alloc_propname(state->interface,
 						      CAST_BAD(attrs[i+1]));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("translatable")))
 		    state->translate_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("context")))
 		    state->context_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("comments")))
-		    state->comment = alloc_propname(state->interface, 
+		    state->comment = glade_xml_alloc_propname(state->interface, 
 						     CAST_BAD(attrs[i+1]));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("agent")))
 		    bad_agent = xmlStrcmp(attrs[i], BAD_CAST("libglade")) != 0;
@@ -608,14 +609,14 @@ glade_parser_start_element(GladeParseState *state,
 	    state->translate_prop = FALSE;
 	    for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 		if (!xmlStrcmp(attrs[i], BAD_CAST("name")))
-		    state->prop_name = alloc_propname(state->interface,
+		    state->prop_name = glade_xml_alloc_propname(state->interface,
 						      CAST_BAD(attrs[i+1]));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("translatable")))
 		    state->translate_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("context")))
 		    state->context_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("comments")))
-		    state->comment = alloc_propname(state->interface, 
+		    state->comment = glade_xml_alloc_propname(state->interface, 
 						     CAST_BAD(attrs[i+1]));
 		else
 		    g_warning("unknown attribute `%s' for <atkproperty>.",
@@ -764,14 +765,14 @@ glade_parser_start_element(GladeParseState *state,
 	    state->translate_prop = FALSE;
 	    for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 		if (!xmlStrcmp(attrs[i], BAD_CAST("name")))
-		    state->prop_name = alloc_propname(state->interface,
+		    state->prop_name = glade_xml_alloc_propname(state->interface,
 						      CAST_BAD(attrs[i+1]));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("translatable")))
 		    state->translate_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("context")))
 		    state->context_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("comments")))
-		    state->comment = alloc_propname(state->interface, 
+		    state->comment = glade_xml_alloc_propname(state->interface, 
 						     CAST_BAD(attrs[i+1]));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("agent")))
 		    bad_agent = xmlStrcmp(attrs[i], BAD_CAST("libglade")) != 0;
@@ -890,7 +891,7 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	prop.has_context  = state->context_prop;
 	prop.translatable = state->translate_prop;
 	prop.comment      = state->comment;
-	prop.value        = alloc_string(state->interface, state->content->str);
+	prop.value        = glade_xml_alloc_string(state->interface, state->content->str);
 
 	g_array_append_val(state->props, prop);
 	state->prop_name = NULL;
@@ -912,7 +913,7 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	prop.has_context  = state->context_prop;
 	prop.translatable = state->translate_prop;
 	prop.comment      = state->comment;
-	prop.value        = alloc_string(state->interface, state->content->str);
+	prop.value        = glade_xml_alloc_string(state->interface, state->content->str);
 
 	g_array_append_val(state->props, prop);
 	state->prop_name = NULL;
@@ -976,7 +977,7 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	prop.has_context  = state->context_prop;
 	prop.translatable = state->translate_prop;
 	prop.comment      = state->comment;
-	prop.value        = alloc_string(state->interface, state->content->str);
+	prop.value        = glade_xml_alloc_string(state->interface, state->content->str);
 
 	g_array_append_val(state->props, prop);
 	state->prop_name = NULL;
@@ -1175,7 +1176,7 @@ glade_parser_parse_file(const gchar *file, const gchar *domain)
     GladeParseState state = { 0 };
 
     if (!g_file_test(file, G_FILE_TEST_IS_REGULAR)) {
-	glade_util_ui_message (glade_default_app_get_window (), 
+	glade_util_ui_message (glade_app_get_window (), 
 			       GLADE_UI_ERROR,
 			       _("Could not find glade file %s"), file);
 	return NULL;
@@ -1188,7 +1189,7 @@ glade_parser_parse_file(const gchar *file, const gchar *domain)
 	state.domain = textdomain(NULL);
 
     if (xmlSAXUserParseFile(&glade_parser, &state, file) < 0) {
-	glade_util_ui_message (glade_default_app_get_window (), 
+	glade_util_ui_message (glade_app_get_window (), 
 			       GLADE_UI_ERROR,
 			       _("Errors parsing glade file %s"), file);
 	if (state.interface)
@@ -1196,7 +1197,7 @@ glade_parser_parse_file(const gchar *file, const gchar *domain)
 	return NULL;
     }
     if (state.state != PARSER_FINISH) {
-	glade_util_ui_message (glade_default_app_get_window (), 
+	glade_util_ui_message (glade_app_get_window (), 
 			       GLADE_UI_ERROR,
 			       _("Errors parsing glade file %s"), file);
 	if (state.interface)
@@ -1247,6 +1248,102 @@ glade_parser_parse_buffer(const gchar *buffer, gint len, const gchar *domain)
     return state.interface;
 }
 
+static gchar *
+modifier_string_from_bits (GdkModifierType modifiers)
+{
+    GString *string = g_string_new ("");
+
+    if (modifiers & GDK_SHIFT_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_SHIFT_MASK");
+    }
+
+    if (modifiers & GDK_LOCK_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_LOCK_MASK");
+    }
+
+    if (modifiers & GDK_CONTROL_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_CONTROL_MASK");
+    }
+
+    if (modifiers & GDK_MOD1_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_MOD1_MASK");
+    }
+
+    if (modifiers & GDK_MOD2_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_MOD2_MASK");
+    }
+
+    if (modifiers & GDK_MOD3_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_MOD3_MASK");
+    }
+
+    if (modifiers & GDK_MOD4_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_MOD4_MASK");
+    }
+
+    if (modifiers & GDK_MOD5_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_MOD5_MASK");
+    }
+
+    if (modifiers & GDK_BUTTON1_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_BUTTON1_MASK");
+    }
+
+    if (modifiers & GDK_BUTTON2_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_BUTTON2_MASK");
+    }
+
+    if (modifiers & GDK_BUTTON3_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_BUTTON3_MASK");
+    }
+
+    if (modifiers & GDK_BUTTON4_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_BUTTON4_MASK");
+    }
+
+    if (modifiers & GDK_BUTTON5_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_BUTTON5_MASK");
+    }
+
+    if (modifiers & GDK_RELEASE_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_RELEASE_MASK");
+    }
+
+    if (string->len > 0)
+	return g_string_free (string, FALSE);
+
+    g_string_free (string, TRUE);
+    return NULL;
+}
+
 static void
 dump_widget(xmlNode *parent, GladeWidgetInfo *info, gint indent)
 {
@@ -1285,7 +1382,9 @@ dump_widget(xmlNode *parent, GladeWidgetInfo *info, gint indent)
 	xmlNodeAddContent(widget, BAD_CAST("\n"));
     }
 
-    if (info->n_atk_props != 0) {
+    if (info->n_atk_props   != 0 ||
+	info->n_atk_actions != 0 ||
+	info->n_relations   != 0) {
 	xmlNode *atk;
 
 	for (j = 0; j < indent + 1; j++)
@@ -1300,7 +1399,7 @@ dump_widget(xmlNode *parent, GladeWidgetInfo *info, gint indent)
 
 	    for (j = 0; j < indent + 2; j++)
 		xmlNodeAddContent(atk, BAD_CAST("  "));
-	    node = xmlNewNode(NULL, BAD_CAST("property"));
+	    node = xmlNewNode(NULL, BAD_CAST("atkproperty"));
 	    xmlSetProp(node, BAD_CAST("name"), BAD_CAST(info->atk_props[i].name));
 	    if (info->atk_props[i].translatable)
 		xmlSetProp(node, BAD_CAST("translatable"), BAD_CAST("yes"));
@@ -1313,6 +1412,35 @@ dump_widget(xmlNode *parent, GladeWidgetInfo *info, gint indent)
 	    xmlAddChild(atk, node);
 	    xmlNodeAddContent(atk, BAD_CAST("\n"));
 	}
+
+	for (i = 0; i < info->n_atk_actions; i++) {
+	    xmlNode *node;
+
+	    for (j = 0; j < indent + 2; j++)
+		xmlNodeAddContent(atk, BAD_CAST("  "));
+	    node = xmlNewNode(NULL, BAD_CAST("atkaction"));
+	    xmlSetProp(node, BAD_CAST("action_name"), 
+		       BAD_CAST(info->atk_actions[i].action_name));
+	    xmlSetProp(node, BAD_CAST("description"), 
+		       BAD_CAST(info->atk_actions[i].description));
+	    xmlAddChild(atk, node);
+	    xmlNodeAddContent(atk, BAD_CAST("\n"));
+	}
+
+	for (i = 0; i < info->n_relations; i++) {
+	    xmlNode *node;
+
+	    for (j = 0; j < indent + 2; j++)
+		xmlNodeAddContent(atk, BAD_CAST("  "));
+	    node = xmlNewNode(NULL, BAD_CAST("atkrelation"));
+	    xmlSetProp(node, BAD_CAST("target"), 
+		       BAD_CAST(info->relations[i].target));
+	    xmlSetProp(node, BAD_CAST("type"), 
+		       BAD_CAST(info->relations[i].type));
+	    xmlAddChild(atk, node);
+	    xmlNodeAddContent(atk, BAD_CAST("\n"));
+	}
+
 	for (j = 0; j < indent + 1; j++)
 	    xmlNodeAddContent(atk, BAD_CAST("  "));
     }
@@ -1338,16 +1466,22 @@ dump_widget(xmlNode *parent, GladeWidgetInfo *info, gint indent)
 
     for (i = 0; i < info->n_accels; i++) {
 	xmlNode *node;
+	gchar   *modifiers;
+
+	modifiers = modifier_string_from_bits (info->accels[i].modifiers);
 
 	for (j = 0; j < indent + 1; j++)
 	    xmlNodeAddContent(widget, BAD_CAST("  "));
 
 	node = xmlNewNode(NULL, BAD_CAST("accelerator"));
 	xmlSetProp(node, BAD_CAST("key"), BAD_CAST(gdk_keyval_name(info->accels[i].key)));
-	xmlSetProp(node, BAD_CAST("modifier"), BAD_CAST("something")/*info->accels[i].modifiers*/);
+	xmlSetProp(node, BAD_CAST("modifiers"), BAD_CAST(modifiers));
 	xmlSetProp(node, BAD_CAST("signal"), BAD_CAST(info->accels[i].signal));
 	xmlAddChild(widget, node);
 	xmlNodeAddContent(widget, BAD_CAST("\n"));
+
+	if (modifiers)
+	    g_free (modifiers);
     }
 
     for (i = 0; i < info->n_children; i++) {
@@ -1411,6 +1545,30 @@ dump_widget(xmlNode *parent, GladeWidgetInfo *info, gint indent)
 	xmlNodeAddContent(widget, BAD_CAST("  "));
 }
 
+static void
+glade_interface_add_comment (xmlDoc *doc)
+{
+	time_t now = time (NULL);
+	xmlNode *comment;
+	gchar *str;
+	
+	str = g_strdup_printf (_(" Generated with %s\n"
+				 "\tVersion: %s\n"
+				 "\tDate: %s"
+				 "\tUser: %s\n"
+				 "\tHost: %s\n"),
+				PACKAGE_NAME,
+				PACKAGE_VERSION,
+				ctime (&now),
+				g_get_user_name (),
+				g_get_host_name ());
+	
+	comment = xmlNewComment(BAD_CAST (str));
+	xmlDocSetRootElement(doc, comment);
+	
+	g_free (str);
+}
+
 static xmlDoc *
 glade_interface_make_doc (GladeInterface *interface)
 {
@@ -1422,6 +1580,9 @@ glade_interface_make_doc (GladeInterface *interface)
     doc->standalone = FALSE;
     xmlCreateIntSubset(doc, BAD_CAST("glade-interface"),
 		       NULL, BAD_CAST("glade-2.0.dtd"));
+
+    glade_interface_add_comment (doc);
+	
     root = xmlNewNode(NULL, BAD_CAST("glade-interface"));
     xmlDocSetRootElement(doc, root);
 
