@@ -85,12 +85,10 @@ glade_popup_paste_cb (GtkMenuItem *item, GladeWidget *widget)
 {
 	glade_util_clear_selection ();
 
-	/* The selected widget is the paste destination
-	 */
-	glade_app_selection_set
-		(glade_widget_get_object (widget), FALSE);
+	/* The selected widget is the paste destination */
+	glade_app_selection_set	(glade_widget_get_object (widget), FALSE);
 
-	glade_app_command_paste ();
+	glade_app_command_paste (NULL);
 }
 
 static void
@@ -125,15 +123,13 @@ glade_popup_add_item_cb (GtkMenuItem *item,
                   PLACEHOLDER POPUP
  *******************************************************/
 static void
-glade_popup_placeholder_paste_cb (GtkMenuItem      *item,
+glade_popup_placeholder_paste_cb (GtkMenuItem *item,
 				  GladePlaceholder *placeholder)
 {
 	glade_util_clear_selection ();
 	glade_app_selection_clear (FALSE);
-
-	/* Set selection on the placeholder first */
-	glade_util_add_selection (GTK_WIDGET (placeholder));
-	glade_app_command_paste ();
+	
+	glade_app_command_paste (placeholder);
 }
 
 
@@ -147,7 +143,7 @@ glade_popup_clipboard_paste_cb (GtkMenuItem      *item,
 	glade_util_clear_selection ();
 	glade_app_selection_clear (FALSE);
 
-	glade_app_command_paste ();
+	glade_app_command_paste (NULL);
 }
 
 static void
@@ -268,6 +264,7 @@ static GtkWidget *
 glade_popup_create_menu (GladeWidget *widget, gboolean add_childs)
 {
 	GtkWidget *popup_menu;
+	gboolean   sensitive;
 
 	popup_menu = gtk_menu_new ();
 
@@ -277,8 +274,11 @@ glade_popup_create_menu (GladeWidget *widget, gboolean add_childs)
 				 glade_popup_cut_cb, widget);
 	glade_popup_append_item (popup_menu, GTK_STOCK_COPY, NULL, TRUE,
 				 glade_popup_copy_cb, widget);
-	glade_popup_append_item (popup_menu, GTK_STOCK_PASTE, NULL, TRUE,
+
+	sensitive = glade_clipboard_get_has_selection (glade_app_get_clipboard ());
+	glade_popup_append_item (popup_menu, GTK_STOCK_PASTE, NULL, sensitive,
 				 glade_popup_paste_cb, widget);
+
 	glade_popup_append_item (popup_menu, GTK_STOCK_DELETE, NULL, TRUE,
 				 glade_popup_delete_cb, widget);
 	glade_popup_append_item (popup_menu, GTK_STOCK_PROPERTIES, NULL, TRUE,
@@ -300,15 +300,17 @@ glade_popup_create_menu (GladeWidget *widget, gboolean add_childs)
 static GtkWidget *
 glade_popup_create_placeholder_menu (GladePlaceholder *placeholder)
 {
-	GtkWidget *popup_menu;
+	GtkWidget   *popup_menu;
 	GladeWidget *parent;
-	
+	gboolean     sensitive;
+
 	popup_menu = gtk_menu_new ();
 
-	glade_popup_append_item (popup_menu, GTK_STOCK_PASTE, NULL, TRUE,
+	sensitive = glade_clipboard_get_has_selection (glade_app_get_clipboard ());
+	glade_popup_append_item (popup_menu, GTK_STOCK_PASTE, NULL, sensitive,
 				 glade_popup_placeholder_paste_cb, placeholder);
 
-	if ((parent = glade_placeholder_get_parent(placeholder)) != NULL)
+	if ((parent = glade_placeholder_get_parent (placeholder)))
 		glade_popup_populate_childs(popup_menu, parent);
 
 	return popup_menu;
