@@ -59,14 +59,19 @@ struct _GladeProjectWindowPriv {
 	GtkActionGroup *recent_actions;	/* Recent projects actions */
 	GtkActionGroup *projects_list_menu_actions;/* Projects list menu actions */
 
-	GQueue *recent_projects;	/* A GtkAction queue */
-	gint rp_max;			/* Maximun Recent Projects entries */
+	GQueue *recent_projects;        /* A GtkAction queue */
+	gint    rp_max;                 /* Maximum Recent Projects entries */
 
 	GtkWindow *palette_window;     /* The window that will contain the palette */
 	GtkWindow *editor_window;      /* The window that will contain the editor */
+	GtkWidget *expand;             /* Expand/Collapse the treeview */
+	GtkWidget *collapse;
+
 };
 
-#define      WINDOW_TITLE                   (_("Glade-3 GUI Builder"))
+
+#define      READONLY_INDICATOR             (_("[read-only]"))
+
 const gint   GLADE_WIDGET_TREE_WIDTH      = 230;
 const gint   GLADE_WIDGET_TREE_HEIGHT     = 300;
 const gint   GLADE_PALETTE_DEFAULT_HEIGHT = 450;
@@ -79,17 +84,24 @@ static void
 gpw_refresh_title (GladeProjectWindow *gpw)
 {
 	GladeProject *active_project;
-	gchar *title, *disp;
+	gchar *title, *project_name;
 
 	active_project = glade_app_get_project ();
 	if (active_project)
 	{
-		disp = glade_project_display_name (active_project, TRUE, FALSE, FALSE);
-		title = g_strdup_printf ("%s - %s", WINDOW_TITLE, disp);
-		g_free (disp);
+		project_name = glade_project_display_name (active_project, TRUE, FALSE, FALSE);
+		
+		if (active_project->readonly != FALSE)
+			title = g_strdup_printf ("%s %s - %s", project_name,
+						 READONLY_INDICATOR, g_get_application_name ());
+		else
+			title = g_strdup_printf ("%s - %s", project_name,
+						 g_get_application_name ());
+		
+		g_free (project_name);
 	}
 	else
-		title = g_strdup_printf ("%s", WINDOW_TITLE);
+		title = g_strdup_printf ("%s", g_get_application_name ());
 
 	gtk_window_set_title (GTK_WINDOW (gpw->priv->window), title);
 	g_free (title);
@@ -139,9 +151,10 @@ gpw_recent_project_add (GladeProjectWindow *gpw, const gchar *project_path)
 	guint merge_id;
 
 	/* Need to check if it's already loaded */
-	if (g_queue_find_custom(gpw->priv->recent_projects, project_path, gpw_rp_cmp))
+	if (g_queue_find_custom (gpw->priv->recent_projects,
+				 project_path, gpw_rp_cmp))
 		return;
-	
+
 	label = glade_util_duplicate_underscores (project_path);
 	if (!label) return;
 	
@@ -244,6 +257,7 @@ gpw_refresh_projects_list_item (GladeProjectWindow *gpw, GladeProject *project)
 {
 	GtkAction *action;
 	gchar *project_name;
+	gchar *tooltip;
 	
 	/* Get associated action */
 	action = GTK_ACTION (g_object_get_data (G_OBJECT (project), "project-list-action"));
@@ -253,8 +267,14 @@ gpw_refresh_projects_list_item (GladeProjectWindow *gpw, GladeProject *project)
 	g_object_set (action, "label", project_name, NULL);
 
 	/* Set action tooltip */
-	g_object_set (action, "tooltip", project->path, NULL);
+	if (project->readonly != FALSE)
+		tooltip = g_strdup_printf ("%s %s", project->path, READONLY_INDICATOR);
+	else
+		tooltip = g_strdup_printf ("%s", project->path);
 	
+	g_object_set (action, "tooltip", tooltip, NULL);
+	
+	g_free (tooltip);
 	g_free (project_name);
 }
 
@@ -380,12 +400,18 @@ gpw_refresh_projects_list_menu (GladeProjectWindow *gpw)
 		GtkRadioAction *action;
 		gchar *action_name;
 		gchar *project_name;
+		gchar *tooltip;
 
 		project = GLADE_PROJECT (l->data);
 
 		action_name = g_strdup_printf ("Project_%d", i);
 		project_name = glade_project_display_name (project, TRUE, TRUE, TRUE);
-		action = gtk_radio_action_new (action_name, project_name, project->path, NULL, i);
+		if (project->readonly != FALSE)
+			tooltip = g_strdup_printf ("%s %s",project->path, READONLY_INDICATOR);
+		else
+			tooltip = g_strdup (project->path);
+
+		action = gtk_radio_action_new (action_name, project_name, tooltip, NULL, i);
 
 		/* Link action and project */
 		g_object_set_data (G_OBJECT (project), "project-list-action", action);
@@ -415,6 +441,7 @@ gpw_refresh_projects_list_menu (GladeProjectWindow *gpw)
 
 		g_free (action_name);
 		g_free (project_name);
+		g_free (tooltip);
 	}
 }
 
@@ -548,7 +575,7 @@ gpw_save (GladeProjectWindow *gpw, GladeProject *project, const gchar *path)
 static void
 gpw_save_as (GladeProjectWindow *gpw, const gchar *dialog_title)
 {
- 	GladeProject *project;
+ 	GladeProject *project, *another_project;
  	GtkWidget *filechooser;
 	gchar *path = NULL;
 	gchar *real_path, *ch;
@@ -588,27 +615,48 @@ gpw_save_as (GladeProjectWindow *gpw, const gchar *dialog_title)
  
  	if (!path)
  		return;
-	
-	ch = strrchr (path, '/');
-	if (!strchr (ch, '.')) 
+
+	ch = strrchr (path, '.');
+	if (!ch || strchr (ch, G_DIR_SEPARATOR))
 		real_path = g_strconcat (path, ".glade", NULL);
 	else 
 		real_path = g_strdup (path);
 	
 	g_free (path);
 
-	if (glade_app_is_project_loaded (real_path))
+	/* checks if selected path is actually writable */
+	if (glade_util_file_is_writeable (real_path) == FALSE)
 	{
-		glade_util_ui_message (gpw->priv->window, GLADE_UI_WARN,
-				       _("%s is already open"), real_path);
+
+		glade_util_ui_message (gpw->priv->window, 
+				       GLADE_UI_ERROR,
+			     	       _("Could not save the file %s. You do not have the permissions necessary to save the file."), 
+				       real_path);
+		g_free (real_path);
 		return;
 	}
+
+	/* checks if another open project is using selected path */
+	if ((another_project = glade_app_get_project_by_path (real_path)) != NULL)
+	{
+		if (project != another_project) {
+
+			glade_util_ui_message (gpw->priv->window, 
+					       GLADE_UI_ERROR,
+				     	       _("Could not save file %s. Another project with that path is open."), 
+					       real_path);
+
+			g_free (real_path);
+			return;
+		}
+
+	}
+
 	gpw_save (gpw, project, real_path);
 
 	g_free (real_path);
 }
 
-
 static void
 gpw_save_cb (GtkAction *action, GladeProjectWindow *gpw)
 {
@@ -825,7 +873,7 @@ gpw_cut_cb (GtkAction *action, GladeProjectWindow *gpw)
 static void
 gpw_paste_cb (GtkAction *action, GladeProjectWindow *gpw)
 {
-	glade_app_command_paste ();
+	glade_app_command_paste (NULL);
 }
 
 static void
@@ -864,22 +912,13 @@ gpw_redo_cb (GtkAction *action, GladeProjectWindow *gpw)
 static gboolean
 gpw_hide_palette_on_delete (GtkWidget *palette, gpointer not_used, GtkUIManager *ui)
 {
-	GtkWidget *palette_item;
-
 	glade_util_hide_window (GTK_WINDOW (palette));
-
-	palette_item = gtk_ui_manager_get_widget (ui, "/MenuBar/ViewMenu/Palette");
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (palette_item), FALSE);
-
-	/* Return true so that the palette is not destroyed */
 	return TRUE;
 }
 
 static void
 gpw_create_palette (GladeProjectWindow *gpw)
 {
-	GtkWidget *palette_item;
-
 	g_return_if_fail (gpw != NULL);
 
 	gpw->priv->palette_window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
@@ -897,15 +936,11 @@ gpw_create_palette (GladeProjectWindow *gpw)
 	/* Delete event, don't destroy it */
 	g_signal_connect (G_OBJECT (gpw->priv->palette_window), "delete_event",
 			  G_CALLBACK (gpw_hide_palette_on_delete), gpw->priv->ui);
-	palette_item = gtk_ui_manager_get_widget (gpw->priv->ui, "/MenuBar/ViewMenu/Palette");
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (palette_item), TRUE);
 }
 
 static void
-gpw_show_palette (GladeProjectWindow *gpw)
+gpw_show_palette (GladeProjectWindow *gpw, gboolean raise)
 {
-	GtkWidget *palette_item;
-
 	g_return_if_fail (gpw != NULL);
 
 	if (gpw->priv->palette_window == NULL)
@@ -913,37 +948,15 @@ gpw_show_palette (GladeProjectWindow *gpw)
 
 	gtk_widget_show (GTK_WIDGET (gpw->priv->palette_window));
 
-	palette_item = gtk_ui_manager_get_widget (gpw->priv->ui,
-						  "/MenuBar/ViewMenu/Palette");
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (palette_item), TRUE);
-}
-
-static void
-gpw_hide_palette (GladeProjectWindow *gpw)
-{
-	GtkWidget *palette_item;
-
-	g_return_if_fail (gpw != NULL);
-
-	glade_util_hide_window (gpw->priv->palette_window);
-
-	palette_item = gtk_ui_manager_get_widget (gpw->priv->ui,
-						  "/MenuBar/ViewMenu/Palette");
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (palette_item), FALSE);
+	if (raise)
+		gtk_window_present (gpw->priv->palette_window);
 }
 
 static gboolean
 gpw_hide_editor_on_delete (GtkWidget *editor, gpointer not_used,
 		GtkUIManager *ui)
 {
-	GtkWidget *editor_item;
-
 	glade_util_hide_window (GTK_WINDOW (editor));
-
-	editor_item = gtk_ui_manager_get_widget (ui, "/MenuBar/ViewMenu/PropertyEditor");
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (editor_item), FALSE);
-
-	/* Return true so that the editor is not destroyed */
 	return TRUE;
 }
 
@@ -964,18 +977,28 @@ gpw_hijack_editor_key_press (GtkWidget          *editor_win,
 	return FALSE;
 }
 
+static void
+gpw_doc_search_cb (GladeEditor        *editor,
+		   const gchar        *book,
+		   const gchar        *page,
+		   const gchar        *search,
+		   GladeProjectWindow *gpw)
+{
+	glade_util_search_devhelp (book, page, search);
+}
+
 static void
 gpw_create_editor (GladeProjectWindow *gpw)
 {
-	GtkWidget *editor_item;
-
 	g_return_if_fail (gpw != NULL);
 
 	gpw->priv->editor_window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
 	gtk_window_set_default_size (GTK_WINDOW (gpw->priv->editor_window), 400, 450);
 
 	gtk_window_set_title  (gpw->priv->editor_window, _("Properties"));
-	gtk_window_move (gpw->priv->editor_window, 350, 0);
+	gtk_window_move (gpw->priv->editor_window, 550, 0);
+
+	gtk_container_set_border_width (GTK_CONTAINER (gpw->priv->editor_window), 6);
 
 	gtk_container_add (GTK_CONTAINER (gpw->priv->editor_window), 
 			   GTK_WIDGET (glade_app_get_editor ()));
@@ -984,21 +1007,14 @@ gpw_create_editor (GladeProjectWindow *gpw)
 	g_signal_connect (G_OBJECT (gpw->priv->editor_window), "delete_event",
 			  G_CALLBACK (gpw_hide_editor_on_delete), gpw->priv->ui);
 
-
 	g_signal_connect (G_OBJECT (gpw->priv->editor_window), "key-press-event",
 			  G_CALLBACK (gpw_hijack_editor_key_press), gpw);
-
-
-	editor_item = gtk_ui_manager_get_widget (gpw->priv->ui,
-						 "/MenuBar/ViewMenu/PropertyEditor");
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (editor_item), TRUE);
 }
 
 static void
 gpw_show_editor (GladeApp *app, gboolean raise)
 {
 	GladeProjectWindow *gpw = GLADE_PROJECT_WINDOW (app);
-	GtkWidget *editor_item;
 
 	g_return_if_fail (GLADE_IS_PROJECT_WINDOW (gpw));
 
@@ -1006,25 +1022,16 @@ gpw_show_editor (GladeApp *app, gboolean raise)
 
 	if (raise)
 		gtk_window_present (gpw->priv->editor_window);
-
-	editor_item = gtk_ui_manager_get_widget (gpw->priv->ui,
-						 "/MenuBar/ViewMenu/PropertyEditor");
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (editor_item), TRUE);
 }
 
 static void
 gpw_hide_editor (GladeApp *app)
 {
 	GladeProjectWindow *gpw = GLADE_PROJECT_WINDOW (app);
-	GtkWidget *editor_item;
 
 	g_return_if_fail (GLADE_IS_PROJECT_WINDOW (gpw));
 
 	glade_util_hide_window (gpw->priv->editor_window);
-
-	editor_item = gtk_ui_manager_get_widget (gpw->priv->ui,
-						 "/MenuBar/ViewMenu/PropertyEditor");
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (editor_item), FALSE);
 }
 
 static void 
@@ -1038,7 +1045,7 @@ gpw_expand_treeview (GtkButton *button, GtkTreeView *tree)
 static GtkWidget* 
 gpw_create_widget_tree_contents (GladeProjectWindow *gpw)
 {
-	GtkWidget *hbox, *vbox, *expand, *collapse;
+	GtkWidget *hbox, *vbox;
 	GladeProjectView *view;
 
 	view = glade_project_view_new (GLADE_PROJECT_VIEW_TREE);
@@ -1057,25 +1064,24 @@ gpw_create_widget_tree_contents (GladeProjectWindow *gpw)
 
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_START);
 
-
-	expand = gtk_button_new_with_mnemonic (_("E_xpand all"));
-	gtk_container_set_border_width (GTK_CONTAINER (expand), 
+	gpw->priv->expand = gtk_button_new_with_mnemonic (_("E_xpand all"));
+	gtk_container_set_border_width (GTK_CONTAINER (gpw->priv->expand), 
 					GLADE_GENERIC_BORDER_WIDTH);
 
-	collapse = gtk_button_new_with_mnemonic (_("_Collapse all"));
-	gtk_container_set_border_width (GTK_CONTAINER (collapse), 
+	gpw->priv->collapse = gtk_button_new_with_mnemonic (_("_Collapse all"));
+	gtk_container_set_border_width (GTK_CONTAINER (gpw->priv->collapse), 
 					GLADE_GENERIC_BORDER_WIDTH);
 
-	gtk_box_pack_start (GTK_BOX (hbox), expand, FALSE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), collapse, FALSE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), gpw->priv->expand, FALSE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), gpw->priv->collapse, FALSE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (view), TRUE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
-	g_signal_connect (G_OBJECT (expand), "clicked",
+	g_signal_connect (G_OBJECT (gpw->priv->expand), "clicked",
 			  G_CALLBACK (gpw_expand_treeview), 
 			  view->tree_view);
 
-	g_signal_connect_swapped (G_OBJECT (collapse), "clicked",
+	g_signal_connect_swapped (G_OBJECT (gpw->priv->collapse), "clicked",
 				  G_CALLBACK (gtk_tree_view_collapse_all), 
 				  view->tree_view);
 
@@ -1086,14 +1092,7 @@ static gboolean
 gpw_hide_clipboard_view_on_delete (GtkWidget *clipboard_view, gpointer not_used,
 				   GtkUIManager *ui)
 {
-	GtkWidget *clipboard_item;
-
 	glade_util_hide_window (GTK_WINDOW (clipboard_view));
-
-	clipboard_item = gtk_ui_manager_get_widget (ui, "/MenuBar/ViewMenu/Clipboard");
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (clipboard_item), FALSE);
-
-	/* return true so that the clipboard view is not destroyed */
 	return TRUE;
 }
 
@@ -1101,22 +1100,17 @@ static void
 gpw_create_clipboard_view (GladeProjectWindow *gpw)
 {
 	GtkWidget *view;
-	GtkWidget *clipboard_item;
 	
 	view = glade_app_get_clipboard_view ();
 
 	g_signal_connect (G_OBJECT (view), "delete_event",
 			  G_CALLBACK (gpw_hide_clipboard_view_on_delete),
 			  gpw->priv->ui);
-	clipboard_item = gtk_ui_manager_get_widget (gpw->priv->ui,
-						    "/MenuBar/ViewMenu/Clipboard");
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (clipboard_item), TRUE);
 }
 
 static void
 gpw_show_clipboard_view (GladeProjectWindow *gpw)
 {
-	GtkWidget *clipboard_item;
 	static gboolean created = FALSE;
 	
 	g_return_if_fail (gpw != NULL);
@@ -1128,78 +1122,86 @@ gpw_show_clipboard_view (GladeProjectWindow *gpw)
 	}
 	
 	gtk_widget_show_all (GTK_WIDGET (glade_app_get_clipboard_view ()));
-
-	clipboard_item = gtk_ui_manager_get_widget (gpw->priv->ui,
-						    "/MenuBar/ViewMenu/Clipboard");
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (clipboard_item), TRUE);
+        gtk_window_present (GTK_WINDOW (glade_app_get_clipboard_view ()));
 }
 
 static void
-gpw_hide_clipboard_view (GladeProjectWindow *gpw)
+gpw_palette_appearance_change_cb (GtkRadioAction *action,
+				  GtkRadioAction *current,
+				  GladeProjectWindow *gpw)
 {
-	GtkWidget *clipboard_item;
+	gint value;
 
-	g_return_if_fail (gpw != NULL);
+	value = gtk_radio_action_get_current_value (action);
 
-	glade_util_hide_window (GTK_WINDOW (glade_app_get_clipboard_view ()));
-
-	clipboard_item = gtk_ui_manager_get_widget (gpw->priv->ui,
-						    "/MenuBar/ViewMenu/Clipboard");
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (clipboard_item), FALSE);
+	glade_palette_set_item_appearance (glade_app_get_palette (), value);
 
 }
 
 static void
-gpw_toggle_palette_cb (GtkAction *action, GladeProjectWindow *gpw)
+gpw_palette_toggle_small_icons_cb (GtkAction *action, GladeProjectWindow *gpw)
 {
-	GtkWidget *palette_item;
- 
-	palette_item = gtk_ui_manager_get_widget (gpw->priv->ui,
-						  "/MenuBar/ViewMenu/Palette");
-
-	if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (palette_item)))
-		gpw_show_palette (gpw);
+	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)))
+		glade_palette_set_use_small_item_icons (glade_app_get_palette(), TRUE);
 	else
-		gpw_hide_palette (gpw);
+		glade_palette_set_use_small_item_icons (glade_app_get_palette(), FALSE);
+
 }
 
 static void
-gpw_toggle_editor_cb (GtkAction *action, GladeProjectWindow *gpw)
+gpw_show_palette_cb (GtkAction *action, GladeProjectWindow *gpw)
 {
-	GtkWidget *editor_item;
-
-	editor_item = gtk_ui_manager_get_widget (gpw->priv->ui,
-						 "/MenuBar/ViewMenu/PropertyEditor");
-
-	if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (editor_item)))
-		gpw_show_editor (GLADE_APP (gpw), FALSE);
-	else
-		gpw_hide_editor (GLADE_APP (gpw));
+	gpw_show_palette (gpw, TRUE);
 }
 
 static void
-gpw_toggle_clipboard_cb (GtkAction *action, GladeProjectWindow *gpw)
+gpw_show_editor_cb (GtkAction *action, GladeProjectWindow *gpw)
 {
-	GtkWidget *clipboard_item;
+	gpw_show_editor (GLADE_APP (gpw), TRUE);
+}
 
-	clipboard_item = gtk_ui_manager_get_widget (gpw->priv->ui,
-						    "/MenuBar/ViewMenu/Clipboard");
-
-	if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (clipboard_item)))
+static void
+gpw_show_clipboard_cb (GtkAction *action, GladeProjectWindow *gpw)
+{
 		gpw_show_clipboard_view (gpw);
-	else
-		gpw_hide_clipboard_view (gpw);
 }
 
-static void gpw_about_cb (GtkAction *action, GladeProjectWindow *gpw)
+static void
+gpw_toggle_editor_help_cb (GtkAction *action, GladeProjectWindow *gpw)
+{
+	GtkWidget *help_item;
+
+	if (glade_util_have_devhelp() == FALSE)
+		return;
+
+	help_item = gtk_ui_manager_get_widget (gpw->priv->ui,
+						 "/MenuBar/ViewMenu/PropertyEditorHelp");
+
+	if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (help_item)))
+		glade_editor_show_context_info (glade_app_get_editor ());
+	else
+		glade_editor_hide_context_info (glade_app_get_editor ());
+}
+
+static void 
+gpw_documentation_cb (GtkAction *action, GladeProjectWindow *gpw)
+{
+	glade_util_search_devhelp ("gladeui", NULL, NULL);
+}
+
+static void 
+gpw_about_cb (GtkAction *action, GladeProjectWindow *gpw)
 {
 	gchar *authors[] =
-		{ "Joaquin Cuenca Abela <e98cuenc@yahoo.com>",
-		  "Chema Celorio <chema@ximian.com>",
+		{ "Chema Celorio <chema@ximian.com>",
+		  "Joaquin Cuenca Abela <e98cuenc@yahoo.com>",
 		  "Paolo Borelli <pborelli@katamail.com>",
 		  "Archit Baweja <bighead@users.sourceforge.net>",
 		  "Shane Butler <shane_b@operamail.com>",
 		  "Tristan Van Berkom <tvb@gnome.org>",
+		  "Ivan Wong <email@ivanwong.info>",
+		  "Juan Pablo Ugarte <juanpablougarte@gmail.com>",
+		  "Vincent Geddes <vincent.geddes@gmail.com>",
 		  NULL };
 
 	gchar *translators =
@@ -1209,12 +1211,25 @@ static void gpw_about_cb (GtkAction *action, GladeProjectWindow *gpw)
 		  "Duarte Loreto <happyguy_pt@hotmail.com>\n"
 		  "Zbigniew Chyla <cyba@gnome.pl>\n"
 		  "Hasbullah Bin Pit <sebol@ikhlas.com>\n"
-		  "Takeshi AIHANA <aihana@gnome.gr.jp>\n");
+		  "Takeshi AIHANA <aihana@gnome.gr.jp>\n"
+		  "Kjartan Maraas <kmaraas@gnome.org>\n"
+		  "Carlos Perell Marn <carlos@gnome-db.org>\n"
+		  "Valek Filippov <frob@df.ru>\n"
+		  "Stanislav Visnovsky <visnovsky@nenya.ms.mff.cuni.cz>\n"
+		  "Christophe Merlet <christophe@merlet.net>\n"
+		  "Funda Wang <fundawang@linux.net.cn>\n"
+		  "Francisco Javier F. Serrador <serrador@cvs.gnome.org>\n"
+		  "Adam Weinberger <adamw@gnome.org>\n"
+		  "Raphael Higino <raphaelh@cvs.gnome.org>\n"
+		  "Clytie Siddall <clytie@riverland.net.au>\n"
+		  "Jordi Mas <jmas@softcatala.org>\n"
+		  "Vincent van Adrighem <adrighem@gnome.org>\n"
+		  "Daniel Nylander <po@danielnylander.se>\n");
 
 	gchar *comments =
-		_("Glade is a User Interface Builder for GTK+ and GNOME.\n"
-		  "This version is a rewrite of the Glade 2.0.0 version, "
-		  "original from Damon Chaplin\n");
+		_("Glade is a User Interface Designer for GTK+ and GNOME.\n"
+		  "This version is a rewrite of the Glade 2 version, "
+		  "originally created by Damon Chaplin\n");
 
 	gchar *license =
 		_("This program is free software; you can redistribute it and/or modify\n"
@@ -1231,11 +1246,13 @@ static void gpw_about_cb (GtkAction *action, GladeProjectWindow *gpw)
 		  "MA 02110-1301, USA.");
 
 	gchar *copyright =
-		"(C) 2001-2005 Ximian, Inc.\n"
-		"(C) 2001-2005 Joaquin Cuenca Abela, Paolo Borelli, et al.";
+		"(C) 2001-2006 Ximian, Inc.\n"
+		"(C) 2001-2006 Joaquin Cuenca Abela, Paolo Borelli, et al.\n"
+		"(C) 2001-2006 Tristan Van Berkom, Juan Pablo Ugarte, et al.";
 	
 	gtk_show_about_dialog (GTK_WINDOW (gpw->priv->window),
 			       "name",  PACKAGE_NAME,
+			       "logo-icon-name", "glade-3",
 			       "authors", authors,
 			       "translator-credits", translators,
 			       "comments", comments,
@@ -1272,14 +1289,25 @@ static const gchar *ui_info =
 "      <menuitem action='Delete'/>\n"
 "    </menu>\n"
 "    <menu action='ViewMenu'>\n"
-"      <menuitem action='Palette'/>\n"
 "      <menuitem action='PropertyEditor'/>\n"
 "      <menuitem action='Clipboard'/>\n"
+"      <menuitem action='Palette'/>\n"
+"      <separator/>\n"
+"      <menu action='PaletteAppearance'>\n"
+"        <menuitem action='IconsAndLabels'/>\n"
+"        <menuitem action='IconsOnly'/>\n"
+"        <menuitem action='LabelsOnly'/>\n"
+"        <separator/>\n"
+"        <menuitem action='UseSmallIcons'/>\n"
+"      </menu>\n"
+"      <menuitem action='PropertyEditorHelp'/>\n"
 "    </menu>\n"
 "    <menu action='ProjectMenu'>\n"
 "      <placeholder name='ProjectsListPlaceholder'/>\n"
 "    </menu>\n"
 "    <menu action='HelpMenu'>\n"
+"      <menuitem action='Manual'/>\n"
+"      <separator/>\n"
 "      <menuitem action='About'/>\n"
 "    </menu>\n"
 "  </menubar>\n"
@@ -1293,82 +1321,113 @@ static const gchar *ui_info =
 "</ui>\n";
 
 static GtkActionEntry static_entries[] = {
-	{ "FileMenu", NULL, "_File" },
-	{ "EditMenu", NULL, "_Edit" },
-	{ "ViewMenu", NULL, "_View" },
-	{ "ProjectMenu", NULL, "_Project" },
-	{ "HelpMenu", NULL, "_Help" },
+	{ "FileMenu", NULL, N_("_File") },
+	{ "EditMenu", NULL, N_("_Edit") },
+	{ "ViewMenu", NULL, N_("_View") },
+	{ "ProjectMenu", NULL, N_("_Project") },
+	{ "HelpMenu", NULL, N_("_Help") },
 	
 	/* FileMenu */
-	{ "New", GTK_STOCK_NEW, "_New", "<control>N",
-	"Create a new project file", G_CALLBACK (gpw_new_cb) },
+	{ "New", GTK_STOCK_NEW, N_("_New"), "<control>N",
+	  N_("Create a new project file"), G_CALLBACK (gpw_new_cb) },
 	
-	{ "Open", GTK_STOCK_OPEN, "_Open","<control>O",
-	"Open a project file", G_CALLBACK (gpw_open_cb) },
+	{ "Open", GTK_STOCK_OPEN, N_("_Open"),"<control>O",
+	  N_("Open a project file"), G_CALLBACK (gpw_open_cb) },
 	
-	{ "Recents", NULL, "Recent Projects", NULL, NULL },
+	{ "Recents", NULL, N_("Open _Recent"), NULL, NULL },
 	
-	{ "ClearRecents", GTK_STOCK_CLEAR, "Clear Recent Projects", NULL,
-	NULL, G_CALLBACK (gpw_recent_project_clear_cb) },
-
-	{ "Quit", GTK_STOCK_QUIT, "_Quit", "<control>Q",
-	"Quit the program", G_CALLBACK (gpw_quit_cb) },
+	{ "ClearRecents", GTK_STOCK_CLEAR, N_("Clear Recent Projects"), NULL,
+	  NULL, G_CALLBACK (gpw_recent_project_clear_cb) },
+	
+	{ "Quit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q",
+	  N_("Quit the program"), G_CALLBACK (gpw_quit_cb) },
 
+	/* ViewMenu */
+	{ "PaletteAppearance", NULL, "Palette _Appearance" },
+	
 	/* HelpMenu */
-	{ "About", GTK_STOCK_ABOUT, "_About", NULL,
-	"Shows the About Dialog", G_CALLBACK (gpw_about_cb) }
+	{ "About", GTK_STOCK_ABOUT, N_("_About"), NULL,
+	  N_("Shows the About Dialog"), G_CALLBACK (gpw_about_cb) },
+
+	{ "Manual", NULL, N_("_Documentation"), NULL,
+	  N_("Documentation about Glade"), G_CALLBACK (gpw_documentation_cb) }
 };
 static guint n_static_entries = G_N_ELEMENTS (static_entries);
 
 static GtkActionEntry project_entries[] = {
 
 	/* FileMenu */
-	{ "Save", GTK_STOCK_SAVE, "_Save","<control>S",
-	"Save the current project file", G_CALLBACK (gpw_save_cb) },
+	{ "Save", GTK_STOCK_SAVE, N_("_Save"),"<control>S",
+	  N_("Save the current project file"), G_CALLBACK (gpw_save_cb) },
 	
-	{ "SaveAs", GTK_STOCK_SAVE_AS, "Save _As...", NULL,
-	"Save the current project file with a different name", G_CALLBACK (gpw_save_as_cb) },
+	{ "SaveAs", GTK_STOCK_SAVE_AS, N_("Save _As..."), NULL,
+	  N_("Save the current project file with a different name"), G_CALLBACK (gpw_save_as_cb) },
 	
-	{ "Close", GTK_STOCK_CLOSE, "_Close", "<control>W",
-	"Close the current project file", G_CALLBACK (gpw_close_cb) },
+	{ "Close", GTK_STOCK_CLOSE, N_("_Close"), "<control>W",
+	  N_("Close the current project file"), G_CALLBACK (gpw_close_cb) },
 
 	/* EditMenu */	
-	{ "Undo", GTK_STOCK_UNDO, "_Undo", "<control>Z",
-	"Undo the last action",	G_CALLBACK (gpw_undo_cb) },
+	{ "Undo", GTK_STOCK_UNDO, N_("_Undo"), "<control>Z",
+	  N_("Undo the last action"),	G_CALLBACK (gpw_undo_cb) },
 	
-	{ "Redo", GTK_STOCK_REDO, "_Redo", "<control>R",
-	"Redo the last action",	G_CALLBACK (gpw_redo_cb) },
+	{ "Redo", GTK_STOCK_REDO, N_("_Redo"), "<control>R",
+	  N_("Redo the last action"),	G_CALLBACK (gpw_redo_cb) },
 	
-	{ "Cut", GTK_STOCK_CUT, "C_ut", NULL,
-	"Cut the selection", G_CALLBACK (gpw_cut_cb) },
+	{ "Cut", GTK_STOCK_CUT, N_("C_ut"), NULL,
+	  N_("Cut the selection"), G_CALLBACK (gpw_cut_cb) },
 	
-	{ "Copy", GTK_STOCK_COPY, "_Copy", NULL,
-	"Copy the selection", G_CALLBACK (gpw_copy_cb) },
+	{ "Copy", GTK_STOCK_COPY, N_("_Copy"), NULL,
+	  N_("Copy the selection"), G_CALLBACK (gpw_copy_cb) },
 	
-	{ "Paste", GTK_STOCK_PASTE, "_Paste", NULL,
-	"Paste the clipboard", G_CALLBACK (gpw_paste_cb) },
+	{ "Paste", GTK_STOCK_PASTE, N_("_Paste"), NULL,
+	  N_("Paste the clipboard"), G_CALLBACK (gpw_paste_cb) },
 	
-	{ "Delete", GTK_STOCK_DELETE, "_Delete", "Delete",
-	"Delete the selection", G_CALLBACK (gpw_delete_cb) }
+	{ "Delete", GTK_STOCK_DELETE, N_("_Delete"), "Delete",
+	  N_("Delete the selection"), G_CALLBACK (gpw_delete_cb) },
+
+	/* ViewMenu */
+
+	{ "PropertyEditor", NULL, N_("Property _Editor"), NULL,
+	  N_("Show the property editor"),
+	  G_CALLBACK (gpw_show_editor_cb) },
+
+	{ "Palette", NULL, N_("_Palette"), NULL,
+	  N_("Show the palette of widgets"),
+	  G_CALLBACK (gpw_show_palette_cb) },
+	
+	{ "Clipboard", NULL, N_("_Clipboard"), NULL,
+	  N_("Show the clipboard"),
+	  G_CALLBACK (gpw_show_clipboard_cb) }
+
+
 };
 static guint n_project_entries = G_N_ELEMENTS (project_entries);
 
 static GtkToggleActionEntry view_entries[] = {
-	/* ViewMenu */
-	{ "Palette", NULL, "_Palette", NULL,
-	"Change the visibility of the palette of widgets",
-	G_CALLBACK (gpw_toggle_palette_cb), TRUE },
 
-	{ "PropertyEditor", NULL, "Property _Editor", NULL,
-	"Change the visibility of the property editor",
-	G_CALLBACK (gpw_toggle_editor_cb), TRUE },
+	{ "UseSmallIcons", NULL, N_("_Use Small Icons"), NULL,
+	  N_("Show items using small icons"),
+	  G_CALLBACK (gpw_palette_toggle_small_icons_cb), FALSE },
 
-	{ "Clipboard", NULL, "_Clipboard", NULL,
-	"Change the visibility of the clipboard",
-	G_CALLBACK (gpw_toggle_clipboard_cb), FALSE }
+	{ "PropertyEditorHelp", NULL, N_("Context _Help"), NULL,
+	  N_("Show or hide contextual help buttons in the editor"),
+	  G_CALLBACK (gpw_toggle_editor_help_cb), FALSE }
 };
 static guint n_view_entries = G_N_ELEMENTS (view_entries);
 
+static GtkRadioActionEntry radio_entries[] = {
+
+	{ "IconsAndLabels", NULL, N_("Text beside icons"), NULL, 
+	  N_("Display items as text beside icons"), GLADE_ITEM_ICON_AND_LABEL },
+
+	{ "IconsOnly", NULL, N_("_Icons only"), NULL, 
+	  N_("Display items as icons only"), GLADE_ITEM_ICON_ONLY },
+
+	{ "LabelsOnly", NULL, N_("_Text only"), NULL, 
+	  N_("Display items as text only"), GLADE_ITEM_LABEL_ONLY },
+};
+static guint n_radio_entries = G_N_ELEMENTS (radio_entries);
+
 static void
 gpw_menu_item_selected_cb (GtkItem *item, GladeProjectWindow *gpw)
 {
@@ -1431,6 +1490,8 @@ gpw_construct_menu (GladeProjectWindow *gpw)
 	GError *error = NULL;
 	
 	gpw->priv->static_actions = gtk_action_group_new (GLADE_ACTION_GROUP_STATIC);
+	gtk_action_group_set_translation_domain (gpw->priv->static_actions, GETTEXT_PACKAGE);
+
 	gtk_action_group_add_actions (gpw->priv->static_actions,
 				      static_entries,
 				      n_static_entries,
@@ -1439,15 +1500,26 @@ gpw_construct_menu (GladeProjectWindow *gpw)
 					     view_entries,
 					     n_view_entries, 
 					     gpw);
+	gtk_action_group_add_radio_actions (gpw->priv->static_actions, radio_entries,
+					    n_radio_entries, GLADE_ITEM_ICON_ONLY,
+					    G_CALLBACK (gpw_palette_appearance_change_cb), gpw);
 
 	gpw->priv->project_actions = gtk_action_group_new (GLADE_ACTION_GROUP_PROJECT);
+	gtk_action_group_set_translation_domain (gpw->priv->project_actions, GETTEXT_PACKAGE);
+
 	gtk_action_group_add_actions (gpw->priv->project_actions,
 				      project_entries,
 				      n_project_entries,
 				      gpw);
 
-	gpw->priv->projects_list_menu_actions = gtk_action_group_new (GLADE_ACTION_GROUP_PROJECTS_LIST_MENU);
+	gpw->priv->projects_list_menu_actions = 
+		gtk_action_group_new (GLADE_ACTION_GROUP_PROJECTS_LIST_MENU);
+	gtk_action_group_set_translation_domain (gpw->priv->projects_list_menu_actions, 
+						 GETTEXT_PACKAGE);
+
 	gpw->priv->recent_actions = gtk_action_group_new (GLADE_ACTION_GROUP_RECENT);
+	gtk_action_group_set_translation_domain (gpw->priv->recent_actions, 
+						 GETTEXT_PACKAGE);
 	
 	gpw->priv->ui = gtk_ui_manager_new ();
 
@@ -1554,14 +1626,12 @@ glade_project_window_create (GladeProjectWindow *gpw)
 	GtkWidget *toolbar;
 	GtkWidget *project_view;
 	GtkWidget *statusbar;
-	gchar *filename;
+	GtkWidget *editor_item, *docs_item;
 
 	app = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_window_move (GTK_WINDOW (app), 0, 0);
 	gtk_window_set_default_size (GTK_WINDOW (app), 300, 450);
-	filename = g_build_filename (glade_icon_dir, "glade-3.png", NULL);
-	gtk_window_set_default_icon_from_file (filename, NULL);
-	g_free (filename);
+
 	gpw->priv->window = app;
 
 	vbox = gtk_vbox_new (FALSE, 0);
@@ -1573,10 +1643,27 @@ glade_project_window_create (GladeProjectWindow *gpw)
 	project_view = gpw_create_widget_tree_contents (gpw);
 	statusbar = gpw_construct_statusbar (gpw);
 
+	gtk_widget_show (menubar);
+	gtk_widget_show (toolbar);
+	gtk_widget_show_all (project_view);
+	gtk_widget_show (statusbar);
+
 	gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, TRUE, 0);
+
+	editor_item = gtk_ui_manager_get_widget (gpw->priv->ui,
+						 "/MenuBar/ViewMenu/PropertyEditorHelp");
+	docs_item = gtk_ui_manager_get_widget (gpw->priv->ui,
+					       "/MenuBar/HelpMenu/Manual");
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (editor_item), FALSE);
+
+	gtk_widget_set_sensitive (editor_item, FALSE);
+	gtk_widget_set_sensitive (docs_item, FALSE);
+
 	gtk_box_pack_start (GTK_BOX (vbox), project_view, TRUE, TRUE, 0);
 	gtk_box_pack_end (GTK_BOX (vbox), statusbar, FALSE, TRUE, 0);
+		
+	gtk_widget_show (vbox);
 
 	gpw_recent_project_config_load (gpw);
 	
@@ -1706,8 +1793,8 @@ gpw_refresh_undo_redo (GladeProjectWindow *gpw)
 
 	if (project != NULL)
 	{
-		undo = glade_command_next_undo_item (project);
-		redo = glade_command_next_redo_item (project);
+		undo = glade_project_next_undo_item (project);
+		redo = glade_project_next_redo_item (project);
 	}
 
 	/* Refresh Undo */
@@ -1747,9 +1834,9 @@ glade_project_window_update_ui (GladeApp *app)
 void
 glade_project_window_show_all (GladeProjectWindow *gpw)
 {
-	gtk_widget_show_all (gpw->priv->window);
-	gpw_show_palette (gpw);
+	gpw_show_palette (gpw, FALSE);
 	gpw_show_editor (GLADE_APP (gpw), FALSE);
+        gtk_widget_show (gpw->priv->window);
 }
 
 static void
@@ -1843,6 +1930,36 @@ glade_project_window_new (void)
 	g_signal_connect (G_OBJECT (glade_app_get_clipboard ()), "notify::has-selection",
 			  G_CALLBACK (gpw_clipboard_notify_handler_cb),
 			  gpw);
-
 	return gpw;
 }
+
+void
+glade_project_window_check_devhelp (GladeProjectWindow *gpw)
+{
+	GtkWidget *editor_item, *docs_item;
+	
+	editor_item = gtk_ui_manager_get_widget (gpw->priv->ui,
+						 "/MenuBar/ViewMenu/PropertyEditorHelp");
+	docs_item = gtk_ui_manager_get_widget (gpw->priv->ui,
+					       "/MenuBar/HelpMenu/Manual");
+
+	if (glade_util_have_devhelp ())
+	{
+		GladeEditor *editor = glade_app_get_editor ();
+		glade_editor_show_info (editor);
+		glade_editor_hide_context_info (editor);
+		
+		g_signal_handlers_disconnect_by_func (editor, gpw_doc_search_cb, gpw);
+		
+		g_signal_connect (editor, "gtk-doc-search",
+				  G_CALLBACK (gpw_doc_search_cb), gpw);
+		
+		gtk_widget_set_sensitive (editor_item, TRUE);
+		gtk_widget_set_sensitive (docs_item, TRUE);
+	}
+	else
+	{
+		gtk_widget_set_sensitive (editor_item, FALSE);
+		gtk_widget_set_sensitive (docs_item, FALSE);
+	}
+}
