@@ -44,11 +44,18 @@
 #include "glade-signal.h"
 #include "glade-parameter.h"
 #include "glade-debug.h"
+#include "glade-fixed.h"
 
 /* hash table that will contain all the GtkWidgetClass'es created, indexed by its name */
 static GHashTable *widget_classes = NULL;
 
-#define GLADE_ICON_SIZE 24
+
+#define GLADE_LARGE_ICON_SUBDIR "22x22"
+#define GLADE_LARGE_ICON_SIZE 22
+
+#define GLADE_SMALL_ICON_SUBDIR "16x16"
+#define GLADE_SMALL_ICON_SIZE 16
+
 
 typedef struct {
 	gchar *parent_name;
@@ -95,6 +102,8 @@ glade_widget_class_free (GladeWidgetClass *widget_class)
 	g_free (widget_class->generic_name);
 	g_free (widget_class->name);
 	g_free (widget_class->catalog);
+	if (widget_class->book)
+		g_free (widget_class->book);
 
 	g_list_foreach (widget_class->properties, (GFunc) glade_property_class_free, NULL);
 	g_list_free (widget_class->properties);
@@ -111,12 +120,21 @@ glade_widget_class_free (GladeWidgetClass *widget_class)
 	g_list_foreach (widget_class->signals, (GFunc) glade_signal_free, NULL);
 	g_list_free (widget_class->signals);
 
-	if (widget_class->icon)
-		g_object_unref (G_OBJECT (widget_class->icon));
+	if (widget_class->cursor != NULL)
+		gdk_cursor_unref (widget_class->cursor);
+
+	if (widget_class->large_icon != NULL)
+		g_object_unref (G_OBJECT (widget_class->large_icon));
+
+	if (widget_class->small_icon != NULL)
+		g_object_unref (G_OBJECT (widget_class->small_icon));
+
 }
 
 static GList *
-gwc_props_from_pspecs (GParamSpec **specs, gint n_specs)
+gwc_props_from_pspecs (GladeWidgetClass  *class,
+		       GParamSpec       **specs,
+		       gint               n_specs)
 {
 	GladePropertyClass *property_class;
 	gint                i;
@@ -125,7 +143,7 @@ gwc_props_from_pspecs (GParamSpec **specs, gint n_specs)
 	for (i = 0; i < n_specs; i++)
 	{
 		if ((property_class = 
-		     glade_property_class_new_from_spec (specs[i])) != NULL)
+		     glade_property_class_new_from_spec (class, specs[i])) != NULL)
 			list = g_list_prepend (list, property_class);
 	}
 	return g_list_reverse (list);
@@ -151,13 +169,18 @@ glade_widget_class_list_properties (GladeWidgetClass *class)
 
 	/* list class properties */
 	specs = g_object_class_list_properties (object_class, &n_specs);
-	list = gwc_props_from_pspecs (specs, n_specs);
+	list = gwc_props_from_pspecs (class, specs, n_specs);
 	g_free (specs);
 
+	/* GtkWidgetClass & descendants have accelerators */
+	if (g_type_is_a (class->type, GTK_TYPE_WIDGET))
+		list = g_list_append (list, glade_property_class_accel_property
+				      (class, class->type));
+
 	/* list the (hard-coded) atk relation properties if applicable */
 	if (glade_util_class_implements_interface (class->type, 
 						   ATK_TYPE_IMPLEMENTOR))
-		atk_list = glade_property_class_list_atk_relations (class->type);
+		atk_list = glade_property_class_list_atk_relations (class, class->type);
 
 	return g_list_concat (list, atk_list);
 }
@@ -186,7 +209,7 @@ glade_widget_class_list_child_properties (GladeWidgetClass *class)
 	}
 
 	specs = gtk_container_class_list_child_properties (object_class, &n_specs);
-	list  = gwc_props_from_pspecs (specs, n_specs);
+	list  = gwc_props_from_pspecs (class, specs, n_specs);
 	g_free (specs);
 
 	/* We have to mark packing properties from GladeWidgetClass
@@ -243,7 +266,7 @@ glade_widget_class_list_signals (GladeWidgetClass *class)
 	guint count;
 	guint *sig_ids;
 	guint num_signals;
-	GladeWidgetClassSignal *cur;
+	GladeSignalClass *cur;
 
 	g_return_val_if_fail (class->type != 0, NULL);
 
@@ -256,11 +279,19 @@ glade_widget_class_list_signals (GladeWidgetClass *class)
 
 			for (count = 0; count < num_signals; count++)
 			{
-				cur = g_new0 (GladeWidgetClassSignal, 1);
-				cur->name = (gchar *) g_signal_name (sig_ids[count]);
+				cur = g_new0 (GladeSignalClass, 1);
+				
+				g_signal_query (sig_ids[count], &(cur->query));
+
+				/* Since glib gave us this signal id... it should
+				 * exist no matter what.
+				 */
+				g_assert (cur->query.signal_id != 0);
+
+				cur->name = (cur->query.signal_name);
 				cur->type = (gchar *) g_type_name (type);
 
-				signals = g_list_append (signals, (GladeWidgetClassSignal *) cur);
+				signals = g_list_prepend (signals, cur);
 			}
 			g_free (sig_ids);
 		}
@@ -268,47 +299,133 @@ glade_widget_class_list_signals (GladeWidgetClass *class)
 		type = g_type_parent (type);
 	}
 
-	return signals;
+	return g_list_reverse (signals);
 }
 
-static GdkPixbuf *
-glade_widget_class_create_icon (GladeWidgetClass *class)
+static void
+glade_widget_class_load_icons (GladeWidgetClass *class)
 {
-	GdkPixbuf *icon  = NULL;
 	GError    *error = NULL;
 	gchar     *icon_path;
 
-	if (class->generic_name)
+
+	/* only certain widget classes need to have icons */
+	if (G_TYPE_IS_INSTANTIATABLE (class->type) == FALSE ||
+            G_TYPE_IS_ABSTRACT (class->type) != FALSE ||
+            class->generic_name == NULL)
 	{
-		icon_path = g_strdup_printf ("%s" G_DIR_SEPARATOR_S "%s.png", 
-					     glade_pixmaps_dir, 
-
-					     class->generic_name);
-		if ((icon = 
-		     gdk_pixbuf_new_from_file (icon_path, &error)) == NULL)
-		{
-			g_warning ("Unable to load icon %s (%s)",
-				   class->name, error->message);
-			error = (g_error_free (error), NULL);
-
-			/* Load missing image to the stock
-			 */
-			if ((icon = gtk_icon_theme_load_icon 
-			     (gtk_icon_theme_get_default (), 
-			      GTK_STOCK_MISSING_IMAGE, 
-			      GLADE_ICON_SIZE, 
-			      GTK_ICON_LOOKUP_USE_BUILTIN, &error)) == NULL)
-			{
-				g_critical ("Unable to load an icon for "
-					    "%s from stock (%s)", 
-					    class->name, 
-					    error->message);
-				error = (g_error_free (error), NULL);
-			}
-		}
-		g_free (icon_path);
+		return;
 	}
-	return icon;
+
+	/* load large 22x22 icon */
+	icon_path = g_strdup_printf ("%s" G_DIR_SEPARATOR_S GLADE_LARGE_ICON_SUBDIR G_DIR_SEPARATOR_S "%s.png", 
+				     glade_pixmaps_dir, 
+				     class->generic_name);
+
+	class->large_icon = gdk_pixbuf_new_from_file_at_size (icon_path, 
+							      GLADE_LARGE_ICON_SIZE, 
+							      GLADE_LARGE_ICON_SIZE, 
+							      &error);
+	
+	if (class->large_icon == NULL)
+	{
+		g_warning (_("Unable to load icon for %s (%s)"), class->name, error->message);
+
+		g_error_free (error);
+		error = NULL;
+
+		/* use stock missing icon */
+		class->large_icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), 
+							      GTK_STOCK_MISSING_IMAGE, 
+						  	      GLADE_LARGE_ICON_SIZE, 
+							      GTK_ICON_LOOKUP_USE_BUILTIN, &error);
+		if (class->large_icon == NULL)
+		{
+			g_critical (_("Unable to load stock icon (%s)"),
+				    error->message);
+
+			g_error_free (error);
+			error = NULL;
+		}
+	}
+	g_free (icon_path);
+
+
+	/* load small 16x16 icon */
+	icon_path = g_strdup_printf ("%s" G_DIR_SEPARATOR_S GLADE_SMALL_ICON_SUBDIR G_DIR_SEPARATOR_S "%s.png", 
+				     glade_pixmaps_dir, 
+				     class->generic_name);
+
+	class->small_icon = gdk_pixbuf_new_from_file_at_size (icon_path, 
+							      GLADE_SMALL_ICON_SIZE, 
+							      GLADE_SMALL_ICON_SIZE, 
+							      &error);
+	
+	if (class->small_icon == NULL)
+	{
+		g_warning (_("Unable to load icon for %s (%s)"), class->name, error->message);
+
+		g_error_free (error);
+		error = NULL;
+
+		/* use stock missing icon */
+		class->small_icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), 
+							      GTK_STOCK_MISSING_IMAGE, 
+						  	      GLADE_SMALL_ICON_SIZE, 
+							      GTK_ICON_LOOKUP_USE_BUILTIN, &error);
+		if (class->small_icon == NULL)
+		{
+			g_critical (_("Unable to load stock icon (%s)"),
+				    error->message);
+
+			g_error_free (error);
+			error = NULL;
+		}
+	}
+
+	g_free (icon_path);
+
+}
+
+static void
+glade_widget_class_create_cursor (GladeWidgetClass *widget_class)
+{
+	GdkPixbuf *tmp_pixbuf;
+	const GdkPixbuf *add_pixbuf;
+	GdkDisplay *display;
+
+	/* only certain widget classes need to have cursors */
+	if (G_TYPE_IS_INSTANTIATABLE (widget_class->type) == FALSE ||
+            G_TYPE_IS_ABSTRACT (widget_class->type) != FALSE ||
+            widget_class->generic_name == NULL)
+		return;
+
+	/* cannot continue if 'add widget' cursor pixbuf has not been loaded for any reason */
+	if ((add_pixbuf = glade_cursor_get_add_widget_pixbuf ()) == NULL)
+		return;
+
+	display = gdk_display_get_default ();
+
+	/* create a temporary pixbuf clear to transparent black*/
+	tmp_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, 32, 32);
+	gdk_pixbuf_fill (tmp_pixbuf, 0x00000000);
+
+
+	/* composite pixbufs */
+	gdk_pixbuf_composite (widget_class->large_icon, tmp_pixbuf,
+			      8, 8, 22, 22,
+			      8, 8, 1, 1,
+                              GDK_INTERP_NEAREST, 255);
+
+	gdk_pixbuf_composite (add_pixbuf, tmp_pixbuf,
+			      0, 0, 12, 12,
+			      0, 0, 1, 1,
+                              GDK_INTERP_NEAREST, 255);
+
+
+	widget_class->cursor = gdk_cursor_new_from_pixbuf (display, tmp_pixbuf, 6, 6);
+
+	g_object_unref(tmp_pixbuf);
 }
 
 static void
@@ -353,7 +470,7 @@ glade_widget_class_update_properties_from_node (GladeXmlNode      *node,
 		}
 		else
 		{
-			property_class = glade_property_class_new ();
+			property_class = glade_property_class_new (widget_class);
 			property_class->id = g_strdup (id);
 			*properties = g_list_append (*properties, property_class);
 			list = g_list_last (*properties);
@@ -563,6 +680,13 @@ glade_widget_class_extend_with_node (GladeWidgetClass *widget_class,
 					      (void **)
 					      &widget_class->launch_editor);
 	}
+	widget_class->fixed = 
+		glade_xml_get_property_boolean (node, GLADE_TAG_FIXED, widget_class->fixed);
+
+	/* Check if this class is toplevel */
+	widget_class->toplevel =
+		glade_xml_get_property_boolean (node, GLADE_XML_TAG_TOPLEVEL,
+						widget_class->toplevel);
 
 	/* if we found a <properties> tag on the xml file, we add the properties
 	 * that we read from the xml file to the class.
@@ -868,6 +992,9 @@ glade_widget_class_merge (GladeWidgetClass *widget_class,
 	g_return_if_fail (GLADE_IS_WIDGET_CLASS (widget_class));
 	g_return_if_fail (GLADE_IS_WIDGET_CLASS (parent_class));
 
+	if (widget_class->fixed == FALSE)
+		widget_class->fixed = parent_class->fixed;
+	
 	if (widget_class->post_create_function == NULL)
 		widget_class->post_create_function = parent_class->post_create_function;
 
@@ -877,6 +1004,9 @@ glade_widget_class_merge (GladeWidgetClass *widget_class,
 	if (widget_class->launch_editor == NULL)
 		widget_class->launch_editor = parent_class->launch_editor;
 
+	if (widget_class->toplevel == FALSE)
+		widget_class->toplevel = parent_class->toplevel;
+	
 	/* merge the parent's properties */
 	glade_widget_class_merge_properties
 		(parent_class->type,
@@ -887,11 +1017,24 @@ glade_widget_class_merge (GladeWidgetClass *widget_class,
 		(&widget_class->children, parent_class->children, widget_class);
 }
 
+/**
+ * glade_widget_class_new:
+ * @class_node: A #GladeXmlNode
+ * @catname: the name of the owning catalog
+ * @library: the name of the library used to load class methods from
+ * @domain: the domain to translate strings from this plugin from
+ * @book: the devhelp search domain for the owning catalog.
+ *
+ * Merges the contents of the @parent_class on the @widget_class.
+ * The properties of the @parent_class will be prepended to
+ * those of @widget_class.
+ */
 GladeWidgetClass *
 glade_widget_class_new (GladeXmlNode *class_node,
 			const gchar  *catname, 
 			const gchar  *library, 
-			const gchar  *domain)
+			const gchar  *domain,
+			const gchar  *book)
 {
 	GladeWidgetClass *widget_class;
 	gchar            *name, *generic_name, *ptr;
@@ -961,6 +1104,10 @@ glade_widget_class_new (GladeXmlNode *class_node,
 	widget_class->generic_name = generic_name;
 	widget_class->palette_name = title;
 	widget_class->type         = glade_util_get_type_from_name (name);
+	widget_class->cursor       = NULL;
+	widget_class->large_icon   = NULL;
+	widget_class->small_icon   = NULL;
+	widget_class->toplevel     = FALSE;
 
 	if (G_TYPE_IS_INSTANTIATABLE (widget_class->type)    &&
 	    G_TYPE_IS_ABSTRACT (widget_class->type) == FALSE &&
@@ -979,6 +1126,9 @@ glade_widget_class_new (GladeXmlNode *class_node,
 	if (strcmp (catname, "gtk+"))
 		widget_class->catalog = g_strdup (catname);
 
+	if (book)
+		widget_class->book = g_strdup (book);
+
 	if (widget_class->type == 0)
 	{
 		glade_widget_class_free (widget_class);
@@ -988,7 +1138,9 @@ glade_widget_class_new (GladeXmlNode *class_node,
 	widget_class->properties = glade_widget_class_list_properties (widget_class);
 	widget_class->signals    = glade_widget_class_list_signals (widget_class);
 	widget_class->children   = glade_widget_class_list_children (widget_class);
-	widget_class->icon       = glade_widget_class_create_icon (widget_class);
+
+	glade_widget_class_load_icons (widget_class);
+	glade_widget_class_create_cursor (widget_class);
 
 	for (parent_type = g_type_parent (widget_class->type);
 	     parent_type != 0;
@@ -1005,8 +1157,7 @@ glade_widget_class_new (GladeXmlNode *class_node,
 		}
 	}
 	
-	if (glade_xml_node_get_children (class_node))
-		glade_widget_class_extend_with_node (widget_class, class_node, domain);
+	glade_widget_class_extend_with_node (widget_class, class_node, domain);
 	
 	/* store the GladeWidgetClass on the cache,
 	 * if it's the first time we store a widget class, then
@@ -1168,6 +1319,84 @@ glade_widget_class_get_child_support  (GladeWidgetClass *class,
 }
 
 
+/**
+ * glade_widget_class_default_params:
+ * @class: a #GladeWidgetClass
+ * @construct: whether to return construct params or not construct params
+ * @n_params: return location if any defaults are specified for this class.
+ * 
+ * Returns: A list of params for use in g_object_newv ()
+ */
+GParameter *
+glade_widget_class_default_params (GladeWidgetClass *class,
+				   gboolean          construct,
+				   guint            *n_params)
+{
+	GArray              *params;
+	GObjectClass        *oclass;
+	GParamSpec         **pspec;
+	GladePropertyClass  *pclass;
+	guint                n_props, i;
+
+	g_return_val_if_fail (GLADE_IS_WIDGET_CLASS (class), NULL);
+	g_return_val_if_fail (n_params != NULL, NULL);
+
+	/* As a slight optimization, we never unref the class
+	 */
+	oclass = g_type_class_ref (class->type);
+	pspec  = g_object_class_list_properties (oclass, &n_props);
+	params = g_array_new (FALSE, FALSE, sizeof (GParameter));
+
+	for (i = 0; i < n_props; i++)
+	{
+		GParameter parameter = { 0, };
+
+		pclass = glade_widget_class_get_property_class
+			(class, pspec[i]->name);
+		
+		/* Ignore properties based on some criteria
+		 */
+		if (pclass == NULL       || /* Unaccounted for in the builder */
+		    pclass->set_function || /* should not be set before 
+					       GladeWidget wrapper exists */
+		    pclass->ignore)         /* Catalog explicitly ignores the object */
+			continue;
+
+		if (construct &&
+		    (pspec[i]->flags & 
+		     (G_PARAM_CONSTRUCT|G_PARAM_CONSTRUCT_ONLY)) == 0)
+			continue;
+		else if (!construct &&
+			 (pspec[i]->flags & 
+			  (G_PARAM_CONSTRUCT|G_PARAM_CONSTRUCT_ONLY)) != 0)
+			continue;
+
+
+		if (g_value_type_compatible (G_VALUE_TYPE (pclass->def),
+					     pspec[i]->value_type) == FALSE)
+		{
+			g_critical ("Type mismatch on %s property of %s",
+				    parameter.name, class->name);
+			continue;
+		}
+
+		if (g_param_values_cmp (pspec[i], 
+					pclass->def, 
+					pclass->orig_def) == 0)
+			continue;
+
+		parameter.name = pspec[i]->name; /* These are not copied/freed */
+		g_value_init (&parameter.value, pspec[i]->value_type);
+		g_value_copy (pclass->def, &parameter.value);
+
+		g_array_append_val (params, parameter);
+	}
+	g_free (pspec);
+
+	*n_params = params->len;
+	return (GParameter *)g_array_free (params, FALSE);
+}
+
 void
 glade_widget_class_container_add (GladeWidgetClass *class,
 				  GObject          *container,
@@ -1420,3 +1649,154 @@ glade_widget_class_get_packing_default (GladeWidgetClass *child_class,
 
 	return NULL;
 }
+
+/**
+ * glade_widget_class_query:
+ * @class: A #GladeWidgetClass
+ *
+ * Returns: whether the user needs to be queried for
+ * certain properties upon creation of this class.
+ */
+gboolean
+glade_widget_class_query (GladeWidgetClass *class)
+{
+	GladePropertyClass *pclass;
+	GList *l;
+
+	for (l = class->properties; l; l = l->next)
+	{
+		pclass = l->data;
+
+		if (pclass->query)
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
+/**
+ * glade_widget_class_create_widget:
+ * @class: a #GladeWidgetClass
+ * @query: whether to display query dialogs if
+ *         applicable to the class
+ * @...: a %NULL terminated list of string/value pairs of #GladeWidget 
+ *       properties
+ *
+ *
+ * This factory function returns a new #GladeWidget of the correct type/class
+ * with the properties defined in @... and queries the user if nescisary.
+ *
+ * The resulting object will have all default properties applied to it
+ * including the overrides specified in the catalog, unless the catalog
+ * has specified 'ignore' for that property.
+ *
+ * Note that the widget class must be fed twice; once as the
+ * leading arg... and also as the property for the #GladeWidget
+ *
+ * this macro returns the newly created #GladeWidget
+ */
+GladeWidget *
+glade_widget_class_create_widget_real (gboolean          query, 
+				       const gchar      *first_property,
+				       ...)
+{
+	GladeWidgetClass *widget_class;
+	GType             gwidget_type;
+	GladeWidget      *gwidget;
+	va_list           vl, vl_copy;
+
+	g_return_val_if_fail (strcmp (first_property, "class") == 0, NULL);
+
+	va_start (vl, first_property);
+	va_copy (vl_copy, vl);
+
+	widget_class = va_arg (vl, GladeWidgetClass *);
+
+	va_end (vl);
+
+	if (GLADE_IS_WIDGET_CLASS (widget_class) == FALSE)
+	{
+		g_critical ("No class found in glade_widget_class_create_widget_real args");
+		va_end (vl_copy);
+		return NULL;
+}
+
+	if (widget_class->fixed)
+		gwidget_type = GLADE_TYPE_FIXED;
+	else 
+		gwidget_type = GLADE_TYPE_WIDGET;
+
+
+	gwidget = (GladeWidget *)g_object_new_valist (gwidget_type,
+						      first_property, 
+						      vl_copy);
+	va_end (vl_copy);
+	
+	if (query && glade_widget_class_query (widget_class))
+	{
+		GladeEditor *editor = glade_app_get_editor ();
+		
+		/* If user pressed cancel on query popup. */
+		if (!glade_editor_query_dialog (editor, gwidget))
+		{
+			g_object_unref (G_OBJECT (gwidget));
+			return NULL;
+		}
+	}
+
+	return gwidget;
+}
+
+
+
+/**
+ * glade_widget_class_create_internal:
+ * @parent:            The parent #GladeWidget, or %NULL for children
+ *                     outside of the hierarchy.
+ * @internal_object:   the #GObject
+ * @internal_name:     a string identifier for this internal widget.
+ * @anarchist:         Whether or not this widget is a widget outside
+ *                     of the parent's hierarchy (like a popup window)
+ * @reason:            The #GladeCreateReason for which this internal widget
+ *                     was created (usually just pass the reason from the post_create
+ *                     function; note also this is used only by the plugin code so
+ *                     pass something usefull here).
+ *
+ * A convenienve function to create a #GladeWidget of the prescribed type
+ * for internal widgets.
+ *
+ * Returns: a freshly created #GladeWidget wrapper object for the
+ *          @internal_object of name @internal_name
+ */
+GladeWidget *
+glade_widget_class_create_internal (GladeWidget      *parent,
+				    GObject          *internal_object,
+				    const gchar      *internal_name,
+				    const gchar      *parent_name,
+				    gboolean          anarchist,
+				    GladeCreateReason reason)
+{
+	GladeWidgetClass *class;
+	GladeProject     *project;
+
+	g_return_val_if_fail (GLADE_IS_WIDGET (parent), NULL);
+	project = glade_widget_get_project (parent);
+
+        if ((class = glade_widget_class_get_by_name 
+	     (G_OBJECT_TYPE_NAME (internal_object))) == NULL)
+	{
+		g_critical ("Unable to find widget class for type %s", 
+			    G_OBJECT_TYPE_NAME (internal_object));
+		return NULL;
+	}
+
+	return glade_widget_class_create_widget (class, FALSE,
+						 "anarchist", anarchist,
+						 "parent", parent,
+						 "project", project,
+						 "internal", internal_name,
+						 "internal-name", parent_name,
+						 "reason", reason,
+						 "object", internal_object,
+						 NULL);
+}
