@@ -70,8 +70,8 @@ GladeXmlContext * glade_xml_context_new_from_path (const gchar *full_path,
 						   const gchar *root_name);
 GladeXmlDoc *     glade_xml_context_get_doc (GladeXmlContext *context);
 
-gchar *		alloc_string   (GladeInterface *interface, const gchar *string);
-gchar *		alloc_propname (GladeInterface *interface, const gchar *string);
+gchar *		glade_xml_alloc_string   (GladeInterface *interface, const gchar *string);
+gchar *		glade_xml_alloc_propname (GladeInterface *interface, const gchar *string);
 
 void            glade_xml_load_sym_from_node (GladeXmlNode     *node_in,
 					      GModule          *module,
