@@ -25,10 +25,10 @@
 /* Circular header dependancie are fixed here with forward 
  * declarations.
  */
-typedef struct _GladeWidget       GladeWidget;
-typedef struct _GladeProperty     GladeProperty;
-typedef struct _GladeProject      GladeProject;
-
+typedef struct _GladeWidget         GladeWidget;
+typedef struct _GladeProperty       GladeProperty;
+typedef struct _GladeProject        GladeProject;
+typedef enum   _GladeItemAppearance GladeItemAppearance;
 
 #include "glade-widget-class.h"
 #include "glade-widget.h"
@@ -42,18 +42,16 @@ typedef struct _GladeProject      GladeProject;
 #include "glade-clipboard.h"
 #include "glade-project-view.h"
 #include "glade-placeholder.h"
-#include "glade-fixed-manager.h"
 #include "glade-utils.h"
 #include "glade-builtins.h"
 #include "glade-xml-utils.h"
-
+#include "glade-fixed.h"
 
 #define GLADE_TAG_FALSE        "False"
 #define GLADE_TAG_TRUE         "True"
 #define GLADE_TAG_YES          "Yes"
 #define GLADE_TAG_NO           "No"
 #define GLADE_ENUM_DATA_TAG    "GladeEnumDataTag"
-#define GLADE_TAG_IN_PALETTE   "InPalette"
 
 #define GLADE_TAG_EVENT_HANDLER_CONNECTED "EventHandlerConnected"
 
@@ -72,6 +70,7 @@ typedef struct _GladeProject      GladeProject;
 #define GLADE_XML_TAG_PACKING  "packing"
 #define GLADE_XML_TAG_PLACEHOLDER "placeholder"
 #define GLADE_XML_TAG_INTERNAL_CHILD "internal-child"
+#define GLADE_XML_TAG_TOPLEVEL "toplevel"
 
 /* Used for catalog tags and attributes */
 #define GLADE_TAG_GLADE_CATALOG                   "glade-catalog"
@@ -80,9 +79,14 @@ typedef struct _GladeProject      GladeProject;
 #define GLADE_TAG_GLADE_WIDGET_GROUP              "glade-widget-group"
 #define GLADE_TAG_GLADE_WIDGET_CLASS_REF          "glade-widget-class-ref"
 
+#define GLADE_TAG_LIBRARY                         "library"
+#define GLADE_TAG_DEPENDS                         "depends"
+#define GLADE_TAG_DOMAIN                          "domain"
+#define GLADE_TAG_BOOK                            "book"
 #define GLADE_TAG_SIGNAL_NAME                     "signal-name"
 #define GLADE_TAG_DEFAULT                         "default"
 #define GLADE_TAG_DISABLED                        "disabled"
+#define GLADE_TAG_DEFAULT_PALETTE_STATE           "default-palette-state"
 #define GLADE_TAG_REPLACE_CHILD_FUNCTION          "replace-child-function"
 #define GLADE_TAG_POST_CREATE_FUNCTION            "post-create-function"
 #define GLADE_TAG_GET_INTERNAL_CHILD_FUNCTION     "get-internal-child-function"
@@ -107,12 +111,10 @@ typedef struct _GladeProject      GladeProject;
 #define GLADE_TAG_OPTIONAL                        "optional"
 #define GLADE_TAG_OPTIONAL_DEFAULT                "optional-default"
 #define GLADE_TAG_VISIBLE                         "visible"
+#define GLADE_TAG_EXPANDED                        "expanded"
 #define GLADE_TAG_GENERIC_NAME                    "generic-name"
 #define GLADE_TAG_NAME                            "name"
 #define GLADE_TAG_TITLE                           "title"
-#define GLADE_TAG_LIBRARY                         "library"
-#define GLADE_TAG_DEPENDS                         "depends"
-#define GLADE_TAG_DOMAIN                          "domain"
 #define GLADE_TAG_ID                              "id"
 #define GLADE_TAG_KEY                             "key"
 #define GLADE_TAG_VALUE                           "value"
@@ -133,18 +135,20 @@ typedef struct _GladeProject      GladeProject;
 #define GLADE_TAG_INIT_FUNCTION                   "init-function"
 #define GLADE_TAG_ATK_ACTION                      "atk-action"
 #define GLADE_TAG_ATK_PROPERTY                    "atk-property"
+#define GLADE_TAG_FIXED                           "fixed"
 
 #define GLADE_NUMERICAL_STEP_INCREMENT             1
 #define GLADE_FLOATING_STEP_INCREMENT              0.01F
 #define GLADE_NUMERICAL_PAGE_INCREMENT             10
 #define GLADE_NUMERICAL_PAGE_SIZE                  1
-#define GLADE_GENERIC_BORDER_WIDTH                 8
+#define GLADE_GENERIC_BORDER_WIDTH                 6
 
 LIBGLADEUI_API gboolean glade_verbose;
 
 LIBGLADEUI_API gchar* glade_pixmaps_dir;
 LIBGLADEUI_API gchar* glade_catalogs_dir;
 LIBGLADEUI_API gchar* glade_modules_dir;
+LIBGLADEUI_API gchar* glade_plugins_dir;
 LIBGLADEUI_API gchar* glade_locale_dir;
 LIBGLADEUI_API gchar* glade_icon_dir;
 
