@@ -31,22 +31,23 @@
 #include <locale.h>
 #include <gmodule.h>
 
-#ifdef G_OS_WIN32
-#include <stdlib.h> /* __argc & __argv on the windows build */
-#endif
+#include <stdlib.h> /* defines __argc & __argv on the windows build */
 
 #include "glade.h"
 #include "glade-project-window.h"
 #include "glade-debug.h"
 
+#define APPLICATION_NAME (_("Glade 3"))
+
 
 /* Application arguments */
-static gboolean version = FALSE;
+static gboolean version = FALSE, without_devhelp = FALSE;
 static gchar **files = NULL;
 
 static GOptionEntry option_entries[] = 
 {
   { "version", '\0', 0, G_OPTION_ARG_NONE, &version, "output version information and exit", NULL },
+  { "without-devhelp", '\0', 0, G_OPTION_ARG_NONE, &without_devhelp, "disable DevHelp check", NULL },
   { G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &files, "", "" },
   { NULL }
 };
@@ -81,8 +82,8 @@ main (int argc, char *argv[])
 	option_context = g_option_context_new ("[FILE...]");
 
 	option_group = g_option_group_new ("glade",
-					   N_("Glade GUI Builder"),
-					   N_("Glade GUI Builder options"),
+					   N_("Glade"),
+					   N_("Glade options"),
 					   NULL, NULL);
 	g_option_group_add_entries (option_group, option_entries);
 	g_option_context_set_main_group (option_context, option_group);
@@ -95,30 +96,38 @@ main (int argc, char *argv[])
 	g_option_context_add_group (option_context, option_group);
 
 	/* Add Gtk option group */
-	g_option_context_add_group (option_context, gtk_get_option_group (TRUE));
+	g_option_context_add_group (option_context, gtk_get_option_group (FALSE));
 
 	/* Parse command line */
 	if (!g_option_context_parse (option_context, &argc, &argv, &error))
 	{
 		g_option_context_free (option_context);
-	
-		g_print ("%s\n", error->message);
-		g_error_free (error);
+		
+		if (error)
+		{
+			g_print ("%s\n", error->message);
+			g_error_free (error);
+		}
+		else
+			g_print ("An unknown error occurred\n");
+
 		return -1;
 	}
 	
+	g_option_context_free (option_context);
+	option_context = NULL;
 
-	if (version == TRUE)
+	if (version != FALSE)
 	{
 		/* Print version information and exit */
 		g_print ("%s\n", PACKAGE_STRING);
 		return 0;
 	}
-	
 
-	g_set_application_name (_("Glade-3 GUI Builder"));
-	
-	glade_setup_log_handlers ();
+	/* Pass NULL here since we parsed the gtk+ args already...
+	 * from this point on we need a DISPLAY variable to be set.
+	 */
+	gtk_init (NULL, NULL);
 	
 	/* Check for gmodule support */
 	if (!g_module_supported ())
@@ -127,9 +136,18 @@ main (int argc, char *argv[])
 			     "for glade to work"));
 		return -1;
 	}
+
+	g_set_application_name (APPLICATION_NAME);
+	gtk_window_set_default_icon_name ("glade-3");
+	
+	glade_setup_log_handlers ();
+	
 	
 	project_window = glade_project_window_new ();
 	
+	if (without_devhelp == FALSE)
+		glade_project_window_check_devhelp (project_window);
+	
 	if (widget_name != NULL)
 	{
 		GladeWidgetClass *class;
@@ -146,20 +164,24 @@ main (int argc, char *argv[])
 		
 		for (i=0; files[i] ; ++i)
 		{
-			if (g_file_test (files[i], G_FILE_TEST_EXISTS) == TRUE)
+			if (g_file_test (files[i], G_FILE_TEST_EXISTS) != FALSE)
 				glade_project_window_open_project (project_window, files[i]);
 			else
 				g_warning (_("Unable to open '%s', the file does not exist.\n"), files[i]);
 		}
 		g_strfreev (files);
 	}
-	else 
+
+	if (glade_app_get_project () == NULL)
 		glade_project_window_new_project (project_window);
 
 	glade_project_window_show_all (project_window);
 
 	gtk_main ();
 
+	/* destroy GladeApp */
+	g_object_unref (project_window);
+
 	return 0;
 }
 
